@@ -379,9 +379,7 @@ static gint number_playlist_second_status_cb(NumberData *number, const gchar *ke
 
 static gint item_balance_status_cb(ItemData *item, const gchar *key, gpointer data)
 {
-	gint n;
-	n = (gfloat)get_balance() / 100.0 * (float)item->sections;
-	return n;
+	return item_section_by_percent(item, (gfloat)get_balance() / 100.0);
 }
 
 static gint item_load_status_cb(ItemData *item, const gchar *key, gpointer data)
@@ -416,9 +414,7 @@ static gint item_mpegversion_status_cb(ItemData *item, const gchar *key, gpointe
 
 static gint item_position_status_cb(ItemData *item, const gchar *key, gpointer data)
 {
-	gint n;
-	n = (float)item->sections * dsp_position;
-	return n;
+	return item_section_by_percent(item, dsp_position);
 }
 
 static gint item_repeat_status_cb(ItemData *item, const gchar *key, gpointer data)
@@ -460,9 +456,7 @@ static gint item_time_mode_status_cb(ItemData *item, const gchar *key, gpointer
 
 static gint item_volume_status_cb(ItemData *item, const gchar *key, gpointer data)
 {
-	gint n;
-	n = (gfloat)get_volume() / 100.0 * (float)item->sections;
-	return n;
+	return item_section_by_percent(item, (gfloat)get_volume() / 100.0);
 }
 
 /* text */
@@ -1076,7 +1070,7 @@ void display_set_loading(gint animate)
 {
 	if (!main_window) return;
 
-	item_animation_step("load", main_window, FALSE, animate);
+	item_animation_step("load", main_window, FALSE, !animate);
 }
 
 void display_set_status(gint delay, gint value)
@@ -1205,6 +1199,7 @@ void display_set_io_information(gint zero)
 	number_value_set("input_rate", main_window, input_bitrate);
 	number_value_set("input_hz", main_window, input_hz / 1000);
 
+	number_value_set("frame", main_window, frames);
 	number_value_set("frame_total", main_window, frames + frames_remaining);
 	calc_times(&h, &dsp_song_minute, &dsp_song_second, seconds + seconds_remaining, FALSE);
 	number_value_set("song_minute", main_window, dsp_song_minute);
@@ -1270,3 +1265,4 @@ void display_set_song_info(void)
 
 
 
+
