@@ -17,8 +17,10 @@
 #include "ui_fileops.h"
 #include "ui_utildlg.h"
 
+#include <fcntl.h>
 #include <errno.h>
 
+
 /* ----------------------------------------------------------
    input / output interface to mpg123
    ----------------------------------------------------------*/
@@ -42,79 +44,113 @@ typedef struct {
 } Mpg123ExtraData;
 
 
-gint mpg123_enabled;
+gint mpg123_enabled = FALSE;
 
 	/* mpg123 options */
-gint		mpg123_buffer_enable = FALSE;
-gint		mpg123_buffer = 512;
-gint		mpg123_downsample = MPG123_DOWNSAMPLE_AUTO;
-gint		mpg123_custom_sample_size = 44100;
-gint		mpg123_mono = FALSE;
-gint		mpg123_8bit = FALSE;
-gint		mpg123_device_enable = FALSE;
-gchar		*mpg123_device = NULL;
-gint		mpg123_aggressive = FALSE;
-gint		mpg123_disable_resync = FALSE;
-gint		mpg123_to_wav = FALSE;
-gint		mpg123_to_raw = FALSE;
-gint		mpg123_to_wav_path_enable = FALSE;
-gchar		*mpg123_to_wav_path = NULL;
-gint		mpg123_read_id3_tags = TRUE;
-gchar *		mpg123_extra_options = NULL;
+gint mpg123_buffer_enable = FALSE;
+gint mpg123_buffer = 512;
+gint mpg123_downsample = MPG123_DOWNSAMPLE_AUTO;
+gint mpg123_custom_sample_size = 44100;
+gint mpg123_mono = FALSE;
+gint mpg123_8bit = FALSE;
+gint mpg123_device_enable = FALSE;
+gchar *mpg123_device = NULL;
+gint mpg123_aggressive = FALSE;
+gint mpg123_disable_resync = FALSE;
+gint mpg123_to_wav = FALSE;
+gint mpg123_to_raw = FALSE;
+gint mpg123_to_wav_path_enable = FALSE;
+gchar *mpg123_to_wav_path = NULL;
+gint mpg123_read_id3_tags = TRUE;
+gchar *mpg123_extra_options = NULL;
 
-static FILE *mpgpipe = NULL;
-static int mypipe[2];
-static gchar pipebuf1[256];
-static gchar *pipebuf_ptr;
-static gint signal_timeout_id = 0;
+/* internal stuff */
+static int mpg123_pipe[2];
+static gint mpg123_pipe_read_id = -1;
+static gint mpg123_data_check_id = -1;
 static gint mpg123_version = MPG123_VER_0_59O;
 
 static SongData *current_sd = NULL;
 
 
-static gint parse_header_info(void);
-static void parse_frame_info(void);
-static void stop_data(void);
-static void song_finished(void);
-static int parse_pipe_buffer(void);
-static int check_pipe_for_data(int fd);
-static gint read_data(gpointer data);
 static void stop_playing_file(void);
 static gint mpg123_extra_is_http(SongData *sd);
 static gchar *http_path_ensure_root(const gchar *path);
 
-static gint parse_header_info(void)
+/*
+ *-----------------------------------------------------------------------------
+ * input parsers
+ *-----------------------------------------------------------------------------
+ */
+
+static gint parse_till_char(gchar **ptr, gchar c)
 {
-	gchar s[128];
+	gchar *start;
+	gchar *end;
+	gchar tmp;
+	gint val;
+
+	start = *ptr;
+	while (*start == ' ') start++;
+	end = start;
+	while (*end != c && *end != '\0') end++;
+
+	tmp = *end;
+	*end = '\0';
+	val = (gint)strtol(start, NULL, 10);
+	*end = tmp;
+
+	*ptr = end;
+
+	return val;
+}
+
+static gint parse_header_info(gchar *text)
+{
+	gchar buf[128];
 	gchar *ptr;
 	gchar *ptr2;
+	gchar *max;
+	gchar *end;
 
-if (debug_mode) printf("h");
+	max = buf + sizeof(buf) - 1;
+	end = text + strlen(text);
 
-	ptr = pipebuf1;
+	if (debug_mode > 2) printf("mpg123 header: ");
+
+	ptr = text;
 
 	/* Version line */
-	if (!strncmp(pipebuf1,"Version",7))
+	if (strncmp(text, "Version", 7) == 0 && text[7] != '\0')
 		{
-if (debug_mode) putchar('V');
+		if (debug_mode > 2) printf("Version\n");
+
 		ptr += 8;
-		ptr2 = s;
-		while (ptr[0] != ' ' && ptr[0] != '\0')
+		ptr2 = buf;
+		while (*ptr != ' ' && *ptr != '\0')
 			{
-			ptr2[0] = ptr[0];
-			ptr2++;
+			*ptr2 = *ptr;
+			if (ptr2 < max) ptr2++;
 			ptr++;
 			}
-		ptr2[0] = '\0';
-		if (strcmp(s, "0.59o") == 0)
+		*ptr2 = '\0';
+		if (strcmp(buf, "0.59o") == 0)
+			{
 			mpg123_version = MPG123_VER_0_59O;
-		else if (strcmp(s, "0.59p") == 0)
+			}
+		else if (strcmp(buf, "0.59p") == 0)
+			{
 			mpg123_version = MPG123_VER_0_59O;
-		else if (strcmp(s, "0.59q") == 0)
+			}
+		else if (strcmp(buf, "0.59q") == 0)
+			{
 			mpg123_version = MPG123_VER_0_59Q;
-		else if (strcmp(s, "0.59r") == 0)
+			}
+		else if (strcmp(buf, "0.59r") == 0)
+			{
 			mpg123_version = MPG123_VER_0_59Q;
-		else if (strncmp(s, "0.59o", 5) < 1)
+			}
+		else if (strncmp(buf, "0.59o", 5) < 1)
 			{
 			static gint warning_done = FALSE;
 			mpg123_version = MPG123_VER_0_59O;
@@ -122,7 +158,7 @@ if (debug_mode) putchar('V');
 				{
 				printf(_("mpg123 version prior to 0.59o found!\n"));
 				printf(_("assuming 0.59o compatible\n"));
-				printf(_("The version line reported was:\n%s\n"), pipebuf1);
+				printf(_("The version line reported was:\n%s\n"), text);
 				}
 			warning_done = TRUE;
 			}
@@ -134,372 +170,285 @@ if (debug_mode) putchar('V');
 				{
 				printf(_("unknown version of mpg123, assuming %s compatible\n"), MPG123_VER_STRING);
 				printf(_("(GQmpeg requires mpg123 0.59o or later)\n"));
-				printf(_("The version line reported was:\n%s\n"), pipebuf1);
+				printf(_("The version line reported was:\n%s\n"), text);
 				}
 			warning_done = TRUE;
 			}
 		if (debug_mode) printf(_("mpg123 version detected: %d\n"), mpg123_version);
+
 		return TRUE;
 		}
 
 	/* MPEG, layer, freq, mode line */
-	if (!strncmp(pipebuf1,"MPEG",4))
+	if (strncmp(text, "MPEG", 4) == 0)
 		{
-if (debug_mode) putchar('M');
+		if (debug_mode > 2) printf("MPEG\n");
+
 		ptr += 4;
-		ptr2 = s;
-		while (ptr[0] != 'L' && ptr[0] != '\0')
+		ptr2 = buf;
+		while (*ptr != 'L' && *ptr != '\0')
 			{
-			if (ptr[0] != ' ' && ptr[0] != ',')
+			if (*ptr != ' ' && *ptr != ',')
 				{
-				ptr2[0] = ptr[0];
-				ptr2++;
+				*ptr2 = *ptr;
+				if (ptr2 < max) ptr2++;
 				}
 			ptr++;
 			}
-		ptr2[0] = '\0';
-		mpeg_version = strtol(s,NULL,10);
+		*ptr2 = '\0';
+		mpeg_version = strtol(buf, NULL, 10);
+
+		if (ptr + 6 > end) return TRUE;
 
 		ptr += 6;
-		ptr2 = s;
-		while (ptr[0] != 'F' && ptr[0] != '\0')
+		ptr2 = buf;
+		while (*ptr != 'F' && *ptr != '\0')
 			{
-			if (ptr[0] != ' ' && ptr[0] != ',')
+			if (*ptr != ' ' && *ptr != ',')
 				{
-				ptr2[0] = ptr[0];
-				ptr2++;
+				*ptr2 = *ptr;
+				if (ptr2 < max) ptr2++;
 				}
 			ptr++;
 			}
-		ptr2[0] = '\0';
-		mpeg_layer = strlen(s);
+		*ptr2 = '\0';
+		mpeg_layer = strlen(buf);
+
+		if (ptr + 5 > end) return TRUE;
 
 		ptr += 5;
-		ptr2 = s;
-		while (ptr[0] != 'm' && ptr[0] != '\0')
+		ptr2 = buf;
+		while (*ptr != 'm' && *ptr != '\0')
 			{
-			if (ptr[0] != ' ' && ptr[0] != ',')
+			if (*ptr != ' ' && *ptr != ',')
 				{
-				ptr2[0] = ptr[0];
-				ptr2++;
+				*ptr2 = *ptr;
+				if (ptr2 < max) ptr2++;
 				}
 			ptr++;
 			}
-		ptr2[0] = '\0';
-		input_hz = strtol(s,NULL,10);
+		*ptr2 = '\0';
+		input_hz = strtol(buf, NULL, 10);
+
+		if (ptr + 5 > end) return TRUE;
 
 		ptr += 5;
-		ptr2 = s;
-		while (ptr[0] != ',' && ptr[0] != '\0')
+		ptr2 = buf;
+		while (*ptr != ',' && *ptr != '\0')
 			{
-			if (ptr[0] != ' ')
+			if (*ptr != ' ')
 				{
-				ptr2[0] = ptr[0];
-				ptr2++;
+				*ptr2 = *ptr;
+				if (ptr2 < max) ptr2++;
 				}
 			ptr++;
 			}
-		ptr2[0] = '\0';
-		if (!strcmp(s,"Stereo")) mpeg_mode = 1;
-		if (!strcmp(s,"Joint-Stereo")) mpeg_mode = 2;
-		if (!strcmp(s,"Dual-Channel")) mpeg_mode = 3;
-		if (!strcmp(s,"Single-Channel")) mpeg_mode = 4;
+		*ptr2 = '\0';
+		if (strcmp(buf, "Stereo") == 0) mpeg_mode = 1;
+		else if (strcmp(buf, "Joint-Stereo") == 0) mpeg_mode = 2;
+		else if (strcmp(buf, "Dual-Channel") == 0) mpeg_mode = 3;
+		else if (strcmp(buf, "Single-Channel") == 0) mpeg_mode = 4;
+
 		return TRUE;
 		}
 
 	/* Channels  line */
-	if (!strncmp(pipebuf1,"Channels:",9))
+	if (strncmp(text, "Channels:", 9) == 0)
 		{
-if (debug_mode) putchar('C');
+		if (debug_mode > 2) printf ("Channels\n");
+
 		ptr += 9;
-		ptr2 = s;
-		while (ptr[0] != ',' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
-		input_channels = strtol(s,NULL,10);
+		input_channels = parse_till_char(&ptr, ',');
+
 		return TRUE;
 		}
 
 	/* Bitrate line */
-	if (!strncmp(pipebuf1,"Bitrate:",8))
+	if (strncmp(text, "Bitrate:", 8) == 0)
 		{
-if (debug_mode) putchar('B');
+		if (debug_mode > 2) printf("Bitrate\n");
+
 		ptr += 8;
-		ptr2 = s;
-		while (ptr[0] != 'K' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
-		input_bitrate = strtol(s,NULL,10);
+		input_bitrate = parse_till_char(&ptr, 'K');
+
 		return TRUE;
 		}
 
 	/* Audio output line */
-	if (!strncmp(pipebuf1,"Audio:",6))
+	if (strncmp(text, "Audio:", 6) == 0)
 		{
-if (debug_mode) putchar('A');
+		if (debug_mode > 2) printf("Audio\n");
+
 		ptr += 6;
-		ptr2 = s;
-		while (ptr[0] != ',' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
-/*		output_conversion = strtol(s,NULL,10); never used anyway */
+		/* output_conversion = (ignored) */
+		parse_till_char(&ptr, ',');
+
+		if (ptr + 7 > end) return TRUE;
 
 		ptr += 7;
-		ptr2 = s;
-		while (ptr[0] != 'e' && ptr[0] != '\0')
+		ptr2 = buf;
+		while (*ptr != 'e' && *ptr != '\0')
 			{
-			if (ptr[0] != ' ' && ptr[0] != ',')
+			if (*ptr != ' ' && *ptr != ',')
 				{
-				ptr2[0] = ptr[0];
-				ptr2++;
+				*ptr2 = *ptr;
+				if (ptr2 < max) ptr2++;
 				}
 			ptr++;
 			}
-		ptr2[0] = '\0';
-		output_hz = strtol(s,NULL,10);
+		*ptr2 = '\0';
+		output_hz = strtol(buf, NULL, 10);
+
+		if (ptr + 9 > end) return TRUE;
 
 		ptr += 9;
-		ptr2 = s;
-		while (ptr[0] != ',' && ptr[0] != '\0')
+		ptr2 = buf;
+		while (*ptr != ',' && *ptr != '\0')
 			{
-			if (ptr[0] >= '0' && ptr[0] <= '9')
+			if (*ptr >= '0' && *ptr <= '9')
 				{
-				ptr2[0] = ptr[0];
-				ptr2++;
+				*ptr2 = *ptr;
+				if (ptr2 < max) ptr2++;
 				}
 			ptr++;
 			}
-		ptr2[0] = '\0';
-		output_bits = strtol(s,NULL,10);
+		*ptr2 = '\0';
+		output_bits = strtol(buf, NULL, 10);
+
+		if (ptr + 11 > end) return TRUE;
 
 		ptr += 11;
-		ptr2 = s;
-		while (ptr[0] != 10 && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
-		output_channels = strtol(s,NULL,10);
+		output_channels = parse_till_char(&ptr, '\n');
 
 		new_song = TRUE;
 		return TRUE;
 		}
+
+	if (debug_mode > 2) printf("not parsed\n");
+
 	return FALSE;
 }
 
-	/* we parse the output to get the current song position
-		including seconds and frames (played and remaining) */
-static void parse_frame_info(void)
+/* parse the output to get the current song position
+ * including seconds and frames (played and remaining)
+ */
+static gint parse_frame_info(gchar *text)
 {
-	gchar frame_cnt[12] = "";
-	gchar frame_cnt_rem [12] = "";
-	gchar sec_cnt [12] = "";
-	gchar sec_cnt_rem [12] = "";
-	gchar min_cnt [12] = "";
-	gchar min_cnt_rem [12] = "";
 	gchar *ptr;
-	gchar *ptr2;
-	
-	if (debug_mode) printf("f");
 
-	ptr = pipebuf1;
-	ptr += 6;
-	ptr2 = frame_cnt;
-	while (ptr[0] != '[' && ptr[0] != '\0')
-		{
-		if (ptr[0] != ' ')
-			{
-			ptr2[0] = ptr[0];
-			ptr2++;
-			}
-		ptr++;
-		}
-	ptr2[0] = '\0';
+	gint frame_cnt;
+	gint frame_cnt_rem;
+	gint sec_cnt;
+	gint sec_cnt_rem;
+	gint min_cnt;
+	gint min_cnt_rem;
+
+	if (strncmp(text, "Frame", 5) != 0 || text[6] == '\0') return FALSE;
+
+	if (debug_mode > 2) printf("Frame\n");
+
+	ptr = text + 6;
+
+	frame_cnt = parse_till_char(&ptr, '[');
+	if (*ptr == '\0') return TRUE;
 	ptr++;
-	ptr2 = frame_cnt_rem;
-	while (ptr[0] != ']' && ptr[0] != '\0')
-		{
-		if (ptr[0] != ' ')
-			{
-			ptr2[0] = ptr[0];
-			ptr2++;
-			}
-		ptr++;
-		}
-	ptr2[0] = '\0';
-	while (ptr[0] != ':' && ptr[0] != '\0')
-		{
-		ptr++;
-		}
+
+	frame_cnt_rem = parse_till_char(&ptr, ']');
+	while (*ptr != ':' && *ptr != '\0') ptr++;
+	if (*ptr == '\0') return TRUE;
 	ptr++;
 
 	if (mpg123_version == MPG123_VER_0_59O) /* 0.00 [000.00] */
 		{
-		ptr2 = sec_cnt;
-		while (ptr[0] != '[' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
+		sec_cnt = parse_till_char(&ptr, '[');
+		if (*ptr == '\0') return TRUE;
 		ptr++;
-		ptr2 = sec_cnt_rem;
-		while (ptr[0] != ']' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
+
+		sec_cnt_rem = parse_till_char(&ptr, ']');
+
+		min_cnt = 0;
+		min_cnt_rem = 0;
 		}
 	else /* 00:00.00 [00:00.00] */
 		{
-		ptr2 = min_cnt;
-		while (ptr[0] != ':' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
+		min_cnt = parse_till_char(&ptr, ':');
+		if (*ptr == '\0') return TRUE;
 		ptr++;
-		ptr2 = sec_cnt;
-		while (ptr[0] != '[' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
+
+		sec_cnt = parse_till_char(&ptr, '[');
+		if (*ptr == '\0') return TRUE;
 		ptr++;
-		ptr2 = min_cnt_rem;
-		while (ptr[0] != ':' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
+
+		min_cnt_rem = parse_till_char(&ptr, ':');
+		if (*ptr == '\0') return TRUE;
 		ptr++;
-		ptr2 = sec_cnt_rem;
-		while (ptr[0] != ']' && ptr[0] != '\0')
-			{
-			if (ptr[0] != ' ')
-				{
-				ptr2[0] = ptr[0];
-				ptr2++;
-				}
-			ptr++;
-			}
-		ptr2[0] = '\0';
+
+		sec_cnt_rem = parse_till_char(&ptr, ']');
 		}
 
-	frames = strtol(frame_cnt,NULL,10);
-	frames_remaining = strtol(frame_cnt_rem,NULL,10);
-	seconds = (strtol(min_cnt,NULL,10) * 60) + strtol(sec_cnt,NULL,10);
-	seconds_remaining = (strtol(min_cnt_rem,NULL,10) * 60) + strtol(sec_cnt_rem,NULL,10);
+	frames = frame_cnt;
+	frames_remaining = frame_cnt_rem;
+	seconds = min_cnt * 60 + sec_cnt;
+	seconds_remaining = min_cnt_rem * 60 + sec_cnt_rem;
 	
-/*	printf("min = [%s] sec = [%s] (%d)\n", min_cnt, sec_cnt, seconds);*/
+	/* printf("min = [%d] sec = [%d] (%d)\n", min_cnt, sec_cnt, seconds); */
+
+	return TRUE;
 }
 
-static void handle_warnings(void)
+static void handle_warnings(gchar *text)
 {
-	if (!strcmp(pipebuf1, "")) return;
+	if (!text || strcmp(text, "") == 0) return;
 	/* normal message output */
-	if (!strncmp(pipebuf1, "High Perf", 9)) return;
-	if (!strncmp(pipebuf1, "Uses code", 9)) return;
-	if (!strncmp(pipebuf1, "THIS SOFT", 9)) return;
-	if (!strncmp(pipebuf1, "Playing M", 9)) return;
+	if (strncmp(text, "High Perf", 9) == 0) return;
+	if (strncmp(text, "Uses code", 9) == 0) return;
+	if (strncmp(text, "THIS SOFT", 9) == 0) return;
+	if (strncmp(text, "Playing M", 9) == 0) return;
 	/* ID3 tags */
-	if (!strncmp(pipebuf1, "Title  :", 8)) return;
-	if (!strncmp(pipebuf1, "Album  :", 8)) return;
-	if (!strncmp(pipebuf1, "Comment:", 8)) return;
+	if (strncmp(text, "Title  :", 8) == 0) return;
+	if (strncmp(text, "Album  :", 8) == 0) return;
+	if (strncmp(text, "Comment:", 8) == 0) return;
 	/* misc output */
-	if (!strncmp(pipebuf1, "Junk at t", 9)) return;
-	if (!strncmp(pipebuf1, "mpg123: Can't rewind stream", 27)) return;
-	if (!strncmp(pipebuf1, "Directory:", 10)) return;
+	if (strncmp(text, "Junk at t", 9) == 0) return;
+	if (strncmp(text, "mpg123: Can't rewind stream", 27) == 0) return;
+	if (strncmp(text, "Directory:", 10) == 0) return;
+	if (strncmp(text, "Getting real-time priority", 26) == 0) return;
 
 	/* for freeamp in mpg123 compat mode */
-	if (!strncmp(pipebuf1, "Copyright", 9)) return;
-	if (!strncmp(pipebuf1, "This soft", 9)) return;
+	if (strncmp(text, "Copyright", 9) == 0) return;
+	if (strncmp(text, "This soft", 9) == 0) return;
 
 	/* ignore lines that begin with an escape sequence (0.59r) */
-	if (pipebuf1[0] == 27 && pipebuf1[1] == ']') return;
+	if (text[0] == 27 && text[1] == ']') return;
 
-	printf(_("Warning, unrecognized mpg123 output:\"%s\")\n"),pipebuf1);
-	if (TRUE)
+	printf(_("Warning, unrecognized mpg123 output:\"%s\"\n"), text);
+
+	if (debug_mode)
 		{
-		if (debug_mode)
+		gchar *ptr = text;
+		while (*ptr != '\0')
 			{
-			gchar *ptr = pipebuf1;
-			while (ptr[0] != '\0')
-				{
-				printf("[%d]", ptr[0]);
-				ptr++;
-				}
-			putchar ('\n');
+			printf("[%d]", *ptr);
+			ptr++;
 			}
+		printf("\n");
 		}
-
 }
 
+static void pipe_reader_shutdown(void)
+{
+	if (mpg123_data_check_id != -1) gtk_timeout_remove(mpg123_data_check_id);
+	mpg123_data_check_id = -1;
+
+	if (mpg123_pipe_read_id != -1) gtk_input_remove(mpg123_pipe_read_id);
+	mpg123_pipe_read_id = -1;
+}
 
 static void stop_data(void)
 {
-	if (signal_timeout_id)
-		{
-		gtk_timeout_remove(signal_timeout_id);
-		signal_timeout_id = 0;
-		}
-	if (mpgpipe)
-		{
-		fclose (mpgpipe);
-		mpgpipe = NULL;
-		}
+	pipe_reader_shutdown();
 	pid = 0;
-if (debug_mode) printf("closed\n");
+	if (debug_mode) printf("mpg123 closed\n");
 }
 
 static void song_finished(void)
@@ -510,40 +459,37 @@ static void song_finished(void)
 }
 
 /* this routine parses the output from mpg123 to check it's current status
-   when the end is detected, 1 or TRUE is returned; any error return -1;
-   a detected frame (second & frame info) return 1 or TRUE; 2 is returned
+   when the end is detected, 1 is returned; any error returns -1;
+   a detected frame (second & frame info) return 1; 2 is returned
    on anything else (including header info) */
-static gint parse_pipe_buffer(void)
+static gint parse_pipe_buffer(gchar *text)
 {
-	/* check for Frame data */
-	if (!strncmp(pipebuf1,"Frame",5))
-		{
-		parse_frame_info();
-		return 1;
-		}
+	if (debug_mode > 1) printf("mpg123 output:\"%s\"\n", text);
+
+	if (parse_frame_info(text)) return 1;
 
 	/* check for end of song (mpg123 reports a line starting
 		with a '[') */
-	if (!strncmp(pipebuf1,"[",1))
+	if (strncmp(text, "[", 1) == 0)
 		{
-		waitpid (pid, NULL, 0);
+		waitpid(pid, NULL, 0);
 		song_finished();
 		return 0;
 		}
 
-	/* we start looking for mpg123 errors here! */
+	/* start looking for mpg123 */
 
 	/* mpg123 errors */
-	if (!strncmp(pipebuf1,"Can't open",10) ||
-	    !strncmp(pipebuf1,"connect",7) ||
-	    strstr(pipebuf1, "file or directory") ||
-	    !strncmp(pipebuf1,"HTTP",4) ||
-	    strstr(pipebuf1, "Device busy") )
+	if (strncmp(text, "Can't open", 10) == 0 ||
+	    strncmp(text, "connect", 7) == 0 ||
+	    strstr(text, "file or directory") ||
+	    strncmp(text, "HTTP", 4) == 0 ||
+	    strstr(text, "Device busy") )
 		{
 		gchar *buf;
 
-		printf(_("Error, mpg123 reported:\"%s\")\n"),pipebuf1);
-		buf = g_strdup_printf("mpg123: %s", pipebuf1);
+		printf(_("Error, mpg123 reported:\"%s\"\n"), text);
+		buf = g_strdup_printf("mpg123: %s", text);
 		warning_dialog(_("GQmpeg: mpg123 error"), buf);
 		g_free(buf);
 
@@ -553,11 +499,12 @@ static gint parse_pipe_buffer(void)
 		return -1;
 		}
 
-	if (!parse_header_info())
+	if (!parse_header_info(text))
 		{
 		/* Unknown output, print it, it may be relevent */
-		handle_warnings();
+		handle_warnings(text);
 		}
+
 	return 2;
 }
 
@@ -569,136 +516,125 @@ static void handle_end_of_file(void)
 	current_sd = NULL;
 }
 
-	/* copied from the Unix programming FAQ */
-static int check_pipe_for_data(int fd)
-     {
-	int rc;
-	fd_set fds;
-	struct timeval tv;
-	
-	FD_ZERO(&fds);
-	FD_SET(fd,&fds);
-	tv.tv_sec = tv.tv_usec = 0;
-
-	rc = select(fd+1, &fds, NULL, NULL, &tv);
-	if (rc < 0)
-		return -1;
-
-	return FD_ISSET(fd,&fds) ? 1 : 0;
+static void handle_read_error(void)
+{
+	printf(_("error reading from mpg123!\n"));
+	stop_playing_file();
+	module_playback_error(current_sd);
+	current_sd = NULL;
 }
 
-static gint read_data(gpointer data)
+static time_t mpg123_data_check_last_time = 0;
+
+static gint mpg123_data_check_update_cb(gpointer data)
 {
-	static int dcheck;
-	int c, d;
-	gint is_http = GPOINTER_TO_INT(data);
+	time_t secs;
+	time_t new_time;
 
-	if (debug_mode) printf("I");
+	secs = (time_t)GPOINTER_TO_INT(data);
+	new_time = time(NULL);
 
-	if (status != STATUS_PLAY) return FALSE;
-	
-	if (!mpgpipe)
-		{
-		mpgpipe = fdopen (mypipe[0], "r");
-
-		if (!mpgpipe)
-			{
-			handle_end_of_file();
-			return FALSE;
-			}
-
-		if (debug_mode) printf("pipe opened\n");
-
-		setvbuf(mpgpipe,NULL,_IONBF,BUFSIZ);
-		pipebuf_ptr = pipebuf1;
-		}
-
-	/* if mpg123 starts sending too much data, we exit this loop
-		after 8k bytes so the program will respond to user
-		input. Otherwise, the program will appear hung.*/
-	d = 8000;
-
-	while (d > 0 && check_pipe_for_data(mypipe[0]) == 1)
-		{
-		c = getc(mpgpipe);
-
-/* to debug mpg123's output
-		if (TRUE) putchar(c)
-*/
-
-		if (c == -1)
-			{
-			handle_end_of_file();
-			return FALSE;
-			}
-
-#if 0
-		/* we ignore escape sequences (for now) */
-		if (c == 27 && getc(mpgpipe) == ']')
-			{
-			gint finished = FALSE;
-			gint over_count = 8; /* don't go forever, may be a broken escape sequence */
-			while (!finished && over_count > 0)
-				{
-				c = getc(mpgpipe);
-				if (c == -1)
-					{
-					handle_end_of_file();
-					return FALSE;
-					}
-				if (c == ';') finished = TRUE;
-				over_count--;
-				}
-			c = getc(mpgpipe);
-			if (c == -1)
-				{
-				handle_end_of_file();
-				return FALSE;
-				}
-			}
-#endif
-
-		if (c == 13 || c == 10 || c== 7) /* the 7 is for 0.59r and it's escape sequences */
-			{
-			pipebuf_ptr[0] = '\0';
-			if (parse_pipe_buffer() == -1) return FALSE;
-			if (status == STATUS_NEXT) return FALSE;
-			pipebuf_ptr = pipebuf1;
-			}
-		else if (pipebuf_ptr - pipebuf1 < 255)
-			{
-			/* um, we ignore extra data beyond 255 chars,
-			   it is probably junk anyway, and may cause
-			   an over-run problem */
-			pipebuf_ptr[0] = c;
-			pipebuf_ptr++;
-			}
-		d--;
-		}
-
-	/* here we check to see if mpg123 sent data within the last
-		10 seconds (66 times through [at 150ms per] with no
-		data). no data means mpg123 encountered an error we
-		did not catch or it crashed
-		NOTE: 20 seconds (132 times) for http. */
-	if ( d == 8000)
-		dcheck++;
-	else
-		dcheck = 0;
-	if (dcheck > 66 && (!is_http || dcheck > 132))
+	if (new_time > mpg123_data_check_last_time + secs)
 		{
 		printf(_("Error: mpg123 stopped sending data! (timeout)\n"));
 		stop_playing_file();
 		module_playback_error(current_sd);
 		current_sd = NULL;
+
+		mpg123_data_check_id = -1;
 		return FALSE;
 		}
 
-	if (debug_mode) printf("O");
-
 	return TRUE;
 }
 
+static void mpg123_data_check_update(gint secs)
+{
+	if (mpg123_data_check_id == -1)
+		{
+		mpg123_data_check_id = gtk_timeout_add(1000, mpg123_data_check_update_cb,
+						       GINT_TO_POINTER(secs));
+		}
+	mpg123_data_check_last_time = time(NULL);
+}
+
+static char mpg123_read_buffer[1024];
+static gint mpg123_read_buffer_pos = 0;
+
+static void mpg123_pipe_read_cb(gpointer data, int fd, GdkInputCondition condition)
+{
+	gint is_http = GPOINTER_TO_INT(data);
+	gchar buf[1024];
+	gint r;
+	gint p;
+	gint l;
+	gchar *ptr;
+
+	if (status != STATUS_PLAY)
+		{
+		pipe_reader_shutdown();
+		return;
+		}
+
+	r = read(fd, buf, sizeof(buf));
+
+	if (debug_mode > 2) printf("mpg123 read input %d bytes\n", r);
+
+	if (r == -1)
+		{
+		handle_read_error();
+		return;
+		}
+
+	if (r == 0)
+		{
+		handle_end_of_file();
+		return;
+                }
+
+	ptr = mpg123_read_buffer + mpg123_read_buffer_pos;
+	l = sizeof(mpg123_read_buffer) - 1;
+	p = 0;
+	while(p < r)
+		{
+		gchar c;
+		c = buf[p];
+		p++;
+
+		if (c == '\r' || c == '\n' || c == 7) /* the 7 is for 0.59r and it's escape sequences */
+			{
+			if (mpg123_read_buffer_pos > 0)
+				{
+				mpg123_read_buffer[mpg123_read_buffer_pos] = '\0';
+				if (parse_pipe_buffer(mpg123_read_buffer) == -1) return;
+				if (status == STATUS_NEXT)
+					{
+					pipe_reader_shutdown();
+					return;
+					}
+				mpg123_read_buffer_pos = 0;
+				}
+			}
+		else if (mpg123_read_buffer_pos < l)
+			{
+			mpg123_read_buffer[mpg123_read_buffer_pos] = c;
+			mpg123_read_buffer_pos++;
+			}
+		}
+
+	/* Check to see if mpg123 sent data within the last
+	 * 10 seconds. No data means mpg123 encountered an error we
+	 * did not catch or it crashed.
+	 * NOTE: 20 seconds (132 times) for http.
+	 */
+	mpg123_data_check_update((is_http) ? 20 : 10);
+}
+
+static void mpg123_input_read_reset(void)
+{
+	mpg123_read_buffer_pos = 0;
+}
+
 #define MPG123_MAX_COMMANDS 32
 
 static gint start_playing_file(SongData *sd, gint position)
@@ -892,7 +828,7 @@ static gint start_playing_file(SongData *sd, gint position)
 
 
 	/* Create the pipe. */
-	if (pipe (mypipe))
+	if (pipe (mpg123_pipe))
 		{
 		fprintf (stderr, _("Pipe failed.\n"));
 		return FALSE;
@@ -905,8 +841,8 @@ static gint start_playing_file(SongData *sd, gint position)
 	if (frk_pid == (pid_t) 0)
 		{
 		/* This is the child process. */
-		dup2(mypipe[1],2);
-		close(mypipe[0]);
+		dup2(mpg123_pipe[1],2);
+		close(mpg123_pipe[0]);
 
 		/* set the group (session) id to this process for future killing */
 		setsid();
@@ -926,12 +862,15 @@ static gint start_playing_file(SongData *sd, gint position)
 		{
 		/* This is the parent process. */
 		pid = (int) frk_pid;
-		close(mypipe[1]);
+		fcntl(mpg123_pipe[0], F_SETFL, O_NONBLOCK);
+		close(mpg123_pipe[1]);
 		}
 
 	if (debug_mode) printf("mpg123 pid = %d\n", pid);
 
-	signal_timeout_id = gtk_timeout_add(150,(GtkFunction) read_data, GINT_TO_POINTER(mpg123_extra_is_http(sd)));
+	mpg123_input_read_reset();
+	mpg123_pipe_read_id = gdk_input_add(mpg123_pipe[0], GDK_INPUT_READ,
+					    mpg123_pipe_read_cb, GINT_TO_POINTER(mpg123_extra_is_http(sd)));
 
 	return TRUE;
 }
@@ -949,7 +888,7 @@ static void stop_playing_file(void)
 		{
 		/* eek!, maybe mpg123 is still in startup, so wait, try again */
 		if (debug_mode) printf("waiting 1 sec to try again: pid = -%d\n", pid);
-		sleep(1); /* is there a portable nanosleep like func? select? */
+		sleep(1);
 		if (kill(-pid, SIGINT) == -1)
 			{
 			printf(_("Failed to successfully send signal to pid = %d\n"), (int)pid);
@@ -961,6 +900,7 @@ static void stop_playing_file(void)
 	if (t != pid && t != -1)
 		{
 		if (debug_mode) printf("second waitpid, 1st returned: %d\n", t);
+		/* this one WILL HANG, so if someone sent a SIGSTOP to mpg123... */
 		waitpid (pid, NULL, 0);
 		if (debug_mode) printf("second waitpid done.\n");
 		}
