@@ -271,9 +271,9 @@ static gint xmp_input_parse(gchar *buffer, gint length)
 
 	/* make it look like a real string, so things are easier */
 	buffer[length] = '\0';
-#if 0
-	printf("xmp output:\"%s\"\n", buffer);
-#endif
+
+	if (debug_mode > 1) printf("xmp output:\"%s\"\n", buffer);
+
 	if (strncmp(buffer, "Tempo[", 6) == 0)
 		{
 		gchar *ptr;
@@ -372,6 +372,7 @@ static void xmp_input_read_cb(gpointer data, int fd, GdkInputCondition condition
 	gchar buf[2048];
 	gint r;
 	gint p;
+	gint l;
 	gchar *ptr;
 
 	r = read(fd, buf, sizeof(buf));
@@ -390,6 +391,7 @@ static void xmp_input_read_cb(gpointer data, int fd, GdkInputCondition condition
 		}
 
 	ptr = xmp_read_buffer + xmp_read_buffer_pos;
+	l = sizeof(xmp_read_buffer) - 1;
 	p = 0;
 	while(p < r)
 		{
@@ -410,7 +412,7 @@ static void xmp_input_read_cb(gpointer data, int fd, GdkInputCondition condition
 			if (!xmp_input_parse(xmp_read_buffer, xmp_read_buffer_len)) return;
 			xmp_read_buffer_len = xmp_read_buffer_pos = 0;
 			}
-		else if (xmp_read_buffer_pos < 2047)
+		else if (xmp_read_buffer_pos < l)
 			{
 			xmp_read_buffer[xmp_read_buffer_pos] = c;
 			xmp_read_buffer_pos++;
@@ -1057,3 +1059,4 @@ void xmp_init(void)
 }
 
 
+
