@@ -942,7 +942,7 @@ int main (int argc, char *argv[])
 	init_vars();
 
 	/* setup random seed for shuffle mode */
-	srand (time (0));
+	srandom (time (NULL));
 
 	/* check that we can find a valid home directory, if not exit semi-gracefully */
 	cbuf = homedir();
@@ -973,8 +973,8 @@ int main (int argc, char *argv[])
 			gchar *command_line_path = g_strconcat(current_path, "/", argv[i], NULL);
 			if (!strcmp(argv[i],"--debug"))
 				{
-				debug_mode = TRUE;
-				printf("debugging enabled\n");
+				debug_mode++;
+				printf("debugging enabled (level %d)\n", debug_mode);
 				}
 			else if ((isdir(command_line_path) || isdir(argv[i])) && !default_current_path)
 				{
@@ -1213,6 +1213,8 @@ int main (int argc, char *argv[])
 	g_free(startup_skin_file);
 	startup_skin_file = NULL;
 
+	main_window_set_title(current_song_get_title());
+
 	if (startup_dockwm) windowmaker_setup_dock();
 
 	if (dock_mode) main_window->allow_move = FALSE;
