@@ -14,13 +14,12 @@
 #define MENUS_H
 
 
-void create_preset_menu(void);
-GtkWidget *create_main_menu(void);
-GtkWidget *create_dir_menu(void);
-GtkWidget *create_file_menu(void);
-GtkWidget *create_file_menu_list(void);
-GtkWidget *create_playlist_menu(void);
-GtkWidget *create_playlist_menu_custom(void);
+GtkWidget *menu_main(void);
+GtkWidget *menu_filelist_dir(void);
+GtkWidget *menu_filelist_file(void);
+GtkWidget *menu_filelist_playlist(void);
+GtkWidget *menu_playlist(void);
+GtkWidget *menu_playlist_custom(void);
 
 
 #endif
