@@ -19,6 +19,8 @@
  *   the BSDs (FreeBSD, NetBSD, OpenBSD)
  *   SPARC Ultra 1 (tested with Solaris 2.5.1)
  *   SGI
+ *   Tru64 (OSF1)
+ *   HPUX (10.X & 11.X) - requires Makefile adjustments, see Note in HPUX section below
  *
  * Each platform interface now has it's own section.
  */
@@ -50,6 +52,14 @@
 #include <audio.h>
 #endif
 
+#if defined(__osf__) && defined(HAVE_OSFMME)
+#include <mme_api.h>
+#endif
+
+#ifdef __hpux
+#include <Alib.h>
+#endif
+
 typedef struct _DeviceData DeviceData;
 struct _DeviceData
 {
@@ -874,6 +884,290 @@ static gint mixer_get_vol(DeviceData *device)
 	return retvol;
 }
 
+/*
+ *--------------------------------------------------------------------
+ * Tru64 (OSF1)
+ *--------------------------------------------------------------------
+ */
+
+#elif defined(__osf__) && defined(HAVE_OSFMME)
+
+#define AUDIO_MAX_GAIN  0x0000FFFF
+#define LEFT_OFFSET             16
+#define RIGHT_OFFSET             0
+
+void mixer_init(gint init_device_id)
+{
+	int i;
+	LPWAVEOUTCAPS waveOutCaps_p = NULL;
+	UINT max_indev  = waveInGetNumDevs();
+	UINT max_outdev = waveOutGetNumDevs();
+
+	if ((max_indev <= 0) && (max_outdev <= 0))
+		{
+		printf(_("no OSF MME devices found\n"));
+		mixer_enabled = FALSE;
+		return;
+		}
+
+	waveOutCaps_p = (WAVEOUTCAPS *)mmeAllocMem(sizeof(WAVEOUTCAPS));
+
+	for (i = 0; i < max_outdev || i < max_indev; ++i)
+		{
+		DeviceData *device = g_new0(DeviceData, 1);
+		device->device_id = i;
+		device->device_name = NULL; /*= device_names[i];*/
+
+		/* may be overwritten below */
+		device->recordable = 0;
+		device->stereo = 0;
+
+		if (i < max_outdev)
+			{
+			waveOutGetDevCaps(i, waveOutCaps_p, sizeof(WAVEOUTCAPS));
+			device->stereo = (waveOutCaps_p->dwSupport & WAVECAPS_LRVOLUME);
+			}
+		if (i < max_indev)
+			{
+			device->recordable = 1;
+			}
+
+		device_list = g_list_append(device_list, device);
+
+		if (debug_mode)
+			{
+			printf("Mixer device added to list: %d, %s, %d, %d\n",
+			       device->device_id, device->device_name,
+			       device->stereo, device->recordable);
+			}
+
+		if (init_device_id == i) current_device = device;
+		}
+
+	mmeFreeMem((char *)waveOutCaps_p);
+
+	if (device_list)
+		{
+		mixer_enabled = TRUE;
+		if (!current_device) current_device = device_list->data;
+		current_vol = mixer_get_vol(current_device);
+		}
+	else
+		{
+		mixer_enabled = FALSE;
+		}
+}
+
+static void mixer_set_vol(DeviceData *device, gint vol)
+{
+	long l, r;
+	DWORD new_vol;
+
+	if (!mixer_enabled) return;
+
+	if (current_bal == 50)
+		{
+		l = vol;
+		r = vol;
+		}
+	else if (current_bal > 50)
+		{
+		l = (float)vol * (100 - current_bal) / 50;
+		r = vol;
+		}
+	else
+		{
+		l = vol;
+		r = (float)vol * current_bal / 50;
+		}
+
+	l = l * AUDIO_MAX_GAIN / 100;
+	r = r * AUDIO_MAX_GAIN / 100;
+	new_vol = (l << LEFT_OFFSET) | (r << RIGHT_OFFSET);
+
+	waveOutSetVolume(device->device_id, new_vol);
+
+	if (debug_mode)
+		{
+		printf("volume set to %3d (0x%04x, 0x%04x = 0x%08x)\n",
+		       vol, l, r, new_vol);
+		}
+}
+
+static gint mixer_get_vol(DeviceData *device)
+{
+	long l=0, r=0;
+	DWORD *vol_p = NULL;
+
+	if (!mixer_enabled) return 0;
+
+	/* get the volume while thrashing the heap */
+	vol_p = (LPDWORD) (mmeAllocMem(sizeof (DWORD)));
+	waveOutGetVolume(device->device_id, vol_p);
+
+	l = ((*vol_p >> LEFT_OFFSET) & AUDIO_MAX_GAIN) * 100 / AUDIO_MAX_GAIN;
+	r = ((*vol_p >> RIGHT_OFFSET)& AUDIO_MAX_GAIN) * 100 / AUDIO_MAX_GAIN;
+
+	if (l == r)
+		{
+		current_bal = 50;
+		}
+	else if (l<r)
+		{
+		current_bal = 50 + (float)(r - l) / r * 50;
+		}
+	else
+		{
+		current_bal = 50 - (float)(l - r) / l * 50;
+		}
+
+	if (debug_mode)
+		{
+		printf("volume is (0x%08x = 0x%04x, 0x%04x)\n", *vol_p, l, r);
+		}
+
+	mmeFreeMem((char *)vol_p);
+
+	if (l > r)
+		{
+		return l;
+		}
+	else
+		{
+		return r;
+		}
+}
+
+/*
+ *--------------------------------------------------------------------
+ * HPUX (10.X & 11.X)
+ *--------------------------------------------------------------------
+ */
+
+#elif defined(__hpux)
+
+/* Note: this requires additions the the Makefile:
+ * add an include path to /opt/audio/include and a libpath
+ * /opt/audio/lib and link with lib libAlib.a.
+ *
+ * TODO: someone should automate this in configure.in...
+ */
+
+void mixer_init(gint init_device_id)
+{
+  Audio *ad = NULL;
+  AGainDB       min;
+  AGainDB       max;
+  AGainDB       cur;
+  DeviceData *device;
+  long  ret;
+  
+  if ((ad = AOpenAudio(NULL, &ret)) == NULL) {
+    perror("Audio");
+    mixer_enabled = FALSE;
+    return ;
+  }
+  
+  max = AMaxOutputGain(ad);
+  min = AMinOutputGain(ad);
+
+  ACloseAudio(ad, &ret);
+
+  device = g_new0(DeviceData, 1);
+  device->device_id = 1;
+  device->device_name = "Default Audio";
+  device->stereo = 1;
+  device_list = g_list_append(device_list, device);
+
+    
+  if (debug_mode) printf("Mixer device added to list: %d, %s, %d\n",
+                          device->device_id, device->device_name,
+                          device->stereo);   
+  current_device = device;
+  
+  current_vol = mixer_get_vol(current_device);
+}
+
+static void mixer_set_vol(DeviceData *device, gint vol)
+{
+  Audio *ad = NULL;
+  AGainDB       min;
+  AGainDB       max;
+  gint          lvol, rvol;
+  AGainDB       lcur, rcur;
+  long  ret;
+  
+  if ((ad = AOpenAudio(NULL, &ret)) == NULL) {
+    perror("Audio");
+    mixer_enabled = FALSE;
+    return ;
+  }
+  
+  max = AMaxOutputGain(ad);
+  min = AMinOutputGain(ad);
+
+
+  if (current_bal < 50) {
+    lvol = vol;
+    rvol = lvol * current_bal / 50;
+  } else if (current_bal > 50) {
+    rvol = vol;
+    lvol = rvol * (100 - current_bal) / 50;
+  } else {
+    lvol = rvol = vol;
+  }
+
+  lcur =  min + (AGainDB)(((float)(max - min)) * lvol/100.0);
+  rcur =  min + (AGainDB)(((float)(max - min)) * rvol/100.0);
+
+  ASetSystemChannelGain(ad, ASGTPlay, ACTLeft, lcur, &ret);
+  ASetSystemChannelGain(ad, ASGTPlay, ACTRight, rcur, &ret);
+
+  ACloseAudio(ad, &ret);
+
+  return;
+}
+
+static gint mixer_get_vol(DeviceData *device)
+{
+  Audio *ad = NULL;
+  AGainDB       min;
+  AGainDB       max;
+  gint          lvol, rvol;
+  AGainDB       lcur, rcur;
+  long  ret;
+  
+  if ((ad = AOpenAudio(NULL, &ret)) == NULL) {
+    perror("Audio");
+    mixer_enabled = FALSE;
+    return ;
+  }
+  
+  max = AMaxOutputGain(ad);
+  min = AMinOutputGain(ad);
+
+  AGetSystemChannelGain(ad, ASGTPlay, ACTLeft, &lcur, &ret);
+  AGetSystemChannelGain(ad, ASGTPlay, ACTRight, &rcur, &ret);
+
+  ACloseAudio(ad, &ret);
+
+  lvol = ((float)(lcur - min)) / ((float)(max - min))*100.0;
+  rvol = ((float)(rcur - min)) / ((float)(max - min))*100.0;
+
+  if (lvol > rvol) {
+    current_bal = rvol * 50 / lvol;
+    return lvol;
+  } else if (lvol < rvol) {
+    current_bal = 100 - (lvol * 50 / rvol);
+    return rvol;
+  } else {
+    current_bal = 50;
+    return lvol;
+  }
+
+  return;
+}
+
 /*
  *--------------------------------------------------------------------
  * Empty functions for unsupported platforms
@@ -976,6 +1270,10 @@ gint get_volume(void)
 	current_vol = mixer_get_vol(current_device);
 #endif
 
+#if defined(__osf__) && defined(HAVE_OSFMME)
+	current_vol = mixer_get_vol(current_device);
+#endif
+
 	return current_vol;
 }
 
@@ -1005,4 +1303,3 @@ gint get_balance(void)
 }
 
 
-
