@@ -71,6 +71,8 @@ static GenericDialog *playlist_save_confirmed(FileDialog *fd, gint overwrite)
 		g_free(buf);
 		generic_dialog_add(gd, _("Overwrite"), playlist_confirm_ok, TRUE);
 
+		gtk_widget_show(gd->dialog);
+
 		return gd;
 		}
 
@@ -160,9 +162,9 @@ static void playlist_save_or_load_dialog(gchar *path, gint type)
 		}
 	else if (type == DIALOG_LOAD)
 		{
-		title = _("Load Playlist");
-		text = _("Load playlist from:");
-		btntext = _("Load");
+		title = _("Open Playlist");
+		text = _("Open playlist from:");
+		btntext = _("Open");
 		hist_key = "playlist_load_save";
 		btnfunc = playlist_dialog_load_cb;
 		}
@@ -492,6 +494,7 @@ void playlist_dialog_load_file(const gchar *path, gint to_playlist)
 	FileDialog *fd;
 	gchar *text;
 	gchar *filter;
+	gint i;
 
 	if (to_playlist)
 		{
@@ -504,12 +507,28 @@ void playlist_dialog_load_file(const gchar *path, gint to_playlist)
 
 	fd = file_dialog_new(_("GQmpeg - open"), text, "GQmpeg", "open",
 			     playlist_dialog_file_close_cb, NULL);
-	file_dialog_add_button(fd, _("Ok"), playlist_dialog_file_load_cb, TRUE);
+	file_dialog_add_button(fd, _("Open"), playlist_dialog_file_load_cb, TRUE);
 
 	filter = filter_get_text_list();
 	file_dialog_add_path_widgets(fd, NULL, path, "file_open", filter, _("Song Files"));
 	g_free(filter);
 
+	i = 0;
+	while (i >= 0)
+		{
+		filter = filter_get_text_list_by_id(i);
+		if (filter)
+			{
+			file_dialog_add_filter(fd, filter, filter_get_description_by_id(i), FALSE);
+			g_free(filter);
+			i++;
+			}
+		else
+			{
+			i = -1;
+			}
+		}
+
 	fd->type = to_playlist;
 
 	gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
