@@ -17,6 +17,7 @@
 #include "display.h"
 #include "dndutil.h"
 #include "filelist.h"
+#include "menus.h"
 #include "playlist.h"
 #include "playlist-dlg.h"
 #include "songinfo.h"
@@ -1107,8 +1108,11 @@ static void dirlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer
 		}
 	if (bevent->button == 3)
 		{
-		gtk_menu_popup (GTK_MENU(dir_menu), NULL, NULL, NULL, NULL,
-				bevent->button, bevent->time);
+		GtkWidget *menu;
+
+		menu = menu_filelist_dir();
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+			       bevent->button, bevent->time);
 		
 		}
 }
@@ -1144,9 +1148,24 @@ static void filelist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointe
 		}
 	if (bevent->button == 3)
 		{
+		GtkWidget *menu;
+		gchar *path;
+
 		gtk_object_set_user_data(GTK_OBJECT(filelist_clist), GINT_TO_POINTER(row));
-		gtk_menu_popup (GTK_MENU(file_menu), NULL, NULL, NULL, NULL,
-				bevent->button, bevent->time);
+
+		path = filelist_get_path(row);
+		if (is_playlist(path))
+			{
+			menu = menu_filelist_playlist();
+			}
+		else
+			{
+			menu = menu_filelist_file();
+			}
+		g_free(path);
+
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+			       bevent->button, bevent->time);
 		}
 
 }
@@ -1182,17 +1201,19 @@ static void playlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointe
 		}
 	if (bevent->button == 3)
 		{
+		GtkWidget *menu;
+
 		gtk_object_set_user_data(GTK_OBJECT(playlist_clist), GINT_TO_POINTER(row));
 		if (playlist_item_is_custom(row))
 			{
-			gtk_menu_popup (GTK_MENU(playlist_menu_custom), NULL, NULL, NULL, NULL,
-				bevent->button, bevent->time);
+			menu = menu_playlist_custom();
 			}
 		else
 			{
-			gtk_menu_popup (GTK_MENU(playlist_menu), NULL, NULL, NULL, NULL,
-				bevent->button, bevent->time);
+			menu = menu_playlist();
 			}
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+			       bevent->button, bevent->time);
 		}
 }
 
@@ -1925,7 +1946,7 @@ static void playlist_window_build(void)
 	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
-	button = gtk_button_new_with_label(_("Load"));
+	button = gtk_button_new_with_label(_("Open"));
 	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) btn_load_playlist_pressed, NULL);
 	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
@@ -2097,4 +2118,3 @@ static void playlist_window_build(void)
 	gtk_widget_show(playlist_window);
 }
 
-
