@@ -68,7 +68,7 @@ static GList *shuffle_list_find_node(gint p)
 
 static void shuffle_list_add (gint p)
 {
-	gint shuffle_p = (float)rand() / RAND_MAX * ( g_list_length(shuffle_list) + 1);
+	gint shuffle_p = (float)random() / RAND_MAX * ( g_list_length(shuffle_list) + 1);
 	GList *node = g_list_nth(playlist, p);
 	if (shuffle_p == g_list_length(shuffle_list))
 		shuffle_list = g_list_append(shuffle_list, node->data);
@@ -181,7 +181,7 @@ void shuffle_list_create(gint start_with_current)
 		node = g_list_nth(work, current_song_get_number());
 		work = g_list_remove(work, node->data);
 
-		p = (float)rand() / RAND_MAX * g_list_length(work);
+		p = (float)random() / RAND_MAX * g_list_length(work);
 		node = g_list_nth(work, p);
 		if (debug_mode) printf("%d ", g_list_index(playlist, node->data));
 		shuffle_list = g_list_prepend(shuffle_list, node->data);
@@ -194,7 +194,7 @@ void shuffle_list_create(gint start_with_current)
 
 	while (work)
 		{
-		gint p =  (float)rand() / RAND_MAX * g_list_length(work);
+		gint p =  (float)random() / RAND_MAX * g_list_length(work);
 		node = g_list_nth(work, p);
 		if (debug_mode) printf("%d ",g_list_index(playlist, node->data));
 		shuffle_list = g_list_prepend(shuffle_list, node->data);
@@ -249,7 +249,7 @@ static gint playlist_load_list_iterate(void)
 /* return 0 on eof, 1 on line parsed, 2 on song added */
 static gint playlist_load_line(FILE *f)
 {
-	gchar s_buf[1025];
+	gchar s_buf[1024];
 	gchar *s_buf_ptr;
 
 	if (debug_mode) putchar('.');
@@ -281,7 +281,7 @@ static gint playlist_load_line(FILE *f)
 			}
 		if (*s_buf=='\n') return 1;
 		s_buf_ptr = s_buf;
-		while (*s_buf_ptr != '\n' && *s_buf_ptr != '\0' && s_buf_ptr < s_buf + strlen(s_buf)) s_buf_ptr++;
+		while (*s_buf_ptr != '\n' && *s_buf_ptr != '\0') s_buf_ptr++;
 		*s_buf_ptr = '\0';
 		if (!playlist_load_strict)
 			{
@@ -1414,7 +1414,7 @@ void playlist_randomize(void)
 		GList *node;
 		gint p;
 
-		p =  (float)rand() / RAND_MAX * g_list_length(list);
+		p =  (float)random() / RAND_MAX * g_list_length(list);
 		node = g_list_nth(list, p);
 		playlist = g_list_prepend(playlist, node->data);
 		list = g_list_remove(list, node->data);
