@@ -16,7 +16,6 @@
 #include "display.h"
 #include "filelist.h"
 #include "ipc.h"
-#include "menus.h"
 #include "mixer.h"
 #include "players.h"
 #include "playlist.h"
@@ -47,7 +46,7 @@ static gint c_mixer_device_id;
 static gint c_wm_decorations;
 static gint c_play_presets;
 static gint c_show_extra_winamp_info;
-static gint c_double_size_mode;
+static gint c_slik_double_size;
 static gint c_title_scrolls_always;
 static gint c_title_show_extension;
 static gint c_title_convert_underscores;
@@ -176,39 +175,11 @@ static gint sort_skin_list_cb(void *a, void *b)
 
 static GList *skin_list_add_dir(GList *list, gchar *skinpath)
 {
-	DIR *dp;
-	struct dirent *dir;
-	struct stat ent_sbuf;
+	GList *dirs;
 
-	if((dp = opendir(skinpath))==NULL)
-		{
-		/* dir not found */
-		return list;
-		}
+	if (!path_list(skinpath, NULL, &dirs)) return list;
 
-	while ((dir = readdir(dp)) != NULL)
-		{
-		/* skips removed files */
-		if (dir->d_ino > 0)
-			{
-			gchar *name;
-			gchar *path;
-
-			name = dir->d_name; 
-			if (strcmp(name, ".") == 0 || strcmp(name, "..") == 0) continue;
-
-			path = g_strconcat(skinpath, "/", name, NULL);
-
-			if (stat(path,&ent_sbuf) >= 0 && S_ISDIR(ent_sbuf.st_mode))
-				{
-				list = g_list_prepend(list, g_strdup(path));
-				}
-			g_free(path);
-			}
-		}
-        closedir(dp);
-
-	return list;
+	return g_list_concat(list, dirs);
 }
 
 static void populate_skin_list(GtkWidget *clist)
@@ -317,7 +288,7 @@ static void apply_configuration_window(void)
 	buf = gtk_entry_get_text(GTK_ENTRY(initial_playlist_entry));
 	if (buf&& strlen(buf) > 0) initial_playlist_pathname = g_strdup(buf);
 
-	if ( double_size_mode != c_double_size_mode || 
+	if ( slik_double_size != c_slik_double_size || 
 		show_extra_winamp_info != c_show_extra_winamp_info) skin_changed = TRUE;
 
 	if (slik_colorshift_on != c_slik_colorshift_on)
@@ -350,7 +321,7 @@ static void apply_configuration_window(void)
 		if (slik_transparency_force) skin_changed = TRUE;
 		}
 
-	double_size_mode = c_double_size_mode;
+	slik_double_size = c_slik_double_size;
 	show_extra_winamp_info = c_show_extra_winamp_info;
 
 	buf = remove_trailing_slash(gtk_entry_get_text(GTK_ENTRY(default_skin_entry)));
@@ -462,7 +433,6 @@ static void apply_configuration_window(void)
 			preset_file[i] = NULL;
 		}
 
-	create_preset_menu();
 	display_presets_refresh();
 	gdk_window_set_decorations (main_window->window->window, wm_decorations);
 }
@@ -515,6 +485,8 @@ static void create_config_window(gint start_tab)
 	gtk_widget_show(configvbox);
 
 	hbox = gtk_hbutton_box_new();
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 0);
+	gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbox), 95, 40);
 	gtk_box_pack_end(GTK_BOX(configvbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
 
@@ -589,7 +561,7 @@ static void create_config_window(gint start_tab)
 	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
-	button = checkbox_new(_("Double sized window"), double_size_mode, &c_double_size_mode);
+	button = checkbox_new(_("Double sized window"), slik_double_size, &c_slik_double_size);
 	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
@@ -1063,7 +1035,7 @@ void show_about_info(void)
 	gtk_window_set_wmclass(GTK_WINDOW(about), "about", "GQmpeg");
 	gtk_signal_connect(GTK_OBJECT(about), "delete_event", GTK_SIGNAL_FUNC(about_delete_cb), NULL);
 
-	gtk_window_set_title(GTK_WINDOW(about), "About - GQmpeg");
+	gtk_window_set_title(GTK_WINDOW(about), _("About - GQmpeg"));
 	gtk_container_border_width(GTK_CONTAINER(about), 5);
 
 	frame = gtk_frame_new(NULL);
