@@ -14,7 +14,6 @@
 #include "preset-dlg.h"
 
 #include "display.h"
-#include "menus.h"
 #include "types.h"
 #include "ui_fileops.h"
 #include "ui_utildlg.h"
@@ -47,7 +46,6 @@ static void preset_dialog_set_cb(GtkWidget *button, gpointer data)
 		preset_name[n] = g_strdup(filename_from_path(fd->dest_path));
 		preset_file[n] = g_strdup(fd->dest_path);
 
-		create_preset_menu();
 		display_presets_refresh();
 		}
 
@@ -88,11 +86,28 @@ void preset_dialog(const gchar *path, gint playlist)
 		{
 		base = NULL;
 		}
-	filter = filter_get_text_list();
-	file_dialog_add_path_widgets(fd, base, path, hist_key, filter, _("Song Files"));
+	file_dialog_add_path_widgets(fd, base, path, hist_key, ".gqmpeg", _("GQmpeg Playlist File"));
 	g_free(base);
+
+	filter = filter_get_text_list();
+	file_dialog_add_filter(fd, filter, _("Song Files"), !playlist);
 	g_free(filter);
-	file_dialog_add_filter(fd, ".gqmpeg", _("GQmpeg Playlist File"), playlist);
+
+	i = 0;
+	while (i >= 0)
+		{
+		filter = filter_get_text_list_by_id(i);
+		if (filter)
+			{
+			file_dialog_add_filter(fd, filter, filter_get_description_by_id(i), FALSE);
+			g_free(filter);
+			i++;
+			}
+		else
+			{
+			i = -1;
+			}
+		}
 
 	hbox = gtk_hbutton_box_new();
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 5);
@@ -122,3 +137,4 @@ void preset_dialog(const gchar *path, gint playlist)
 }
 
 
+
