@@ -21,6 +21,8 @@ void add_to_filter(const gchar *text, const gchar *description, gint module_id);
 void remove_from_filter(const gchar *text);
 gint filter_determine_module_id(const gchar *path);
 gchar *filter_get_text_list(void);
+gchar *filter_get_text_list_by_id(gint module_id);
+const gchar *filter_get_description_by_id(gint module_id);
 
 void add_to_typelist(const gchar *format, const gchar *description, gint module_id, gint live,
 		     gint (*is_type_func)(const gchar *),
