@@ -274,8 +274,13 @@ static GdkPixbuf *dial_get_handle_pixbuf(DialData *dial, gint pressed, gint prel
 	return dial->pb_normal;
 }
 
-static void dial_force_item_draw_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui)
+static void dial_force_item_draw_cb(WidgetData *wd, gpointer data, UIData *ui)
 {
+	DialData *dial = data;
+	ItemData *item = wd->widget;
+
+	item->current = item_section_by_percent(item, dial->value);
+
 	wd->hidden = FALSE;
 	ui_widget_draw(ui, wd, FALSE, TRUE);
 	wd->hidden = TRUE;
@@ -298,9 +303,8 @@ static void real_dial_render(DialData *dial, GdkPixbuf *pb, UIData *ui)
 
 	if (dial->item_key)
 		{
-		item_section_set_by_percent(dial->item_key, ui, dial->value);
-		skin_widget_for_each_key(ui, dial->item_key, item_type_id(),
-					 dial_force_item_draw_cb, NULL);
+		ui_widget_for_each_key(ui, dial->item_key, item_type_id(),
+				       dial_force_item_draw_cb, dial);
 		}
 
 	handle = dial_get_handle_pixbuf(dial, dial->pushed, dial->prelit);
