@@ -2539,12 +2539,7 @@ static void edit_load_cb(GtkWidget *button, gpointer data)
 
 	if (skin)
 		{
-		ui_skin_set(ed->ui, skin);
-		g_free(ed->ui->skin_path);
-		ed->ui->skin_path = g_strdup(skin_path);
-
-		g_free(ed->ui->skin_mode_key);
-		ed->ui->skin_mode_key = g_strdup(skin_key);
+		ui_skin_set(ed->ui, skin, skin_path, skin_key);
 
 		ed->active_widget = NULL;
 		prop_clear(ed);
@@ -2677,7 +2672,10 @@ static void ui_edit_copy_registry(UIData *ui, UIData *source)
 			    (rd->type == button_type_id() &&
 				(strcmp(srd->key, "exit") == 0 ||	/* disable keys that should 	*/
 				strcmp(srd->key, "iconify") == 0 ||	/* not work in the editor	*/
-				strcmp(srd->key, "skin_toggle") == 0) ) )
+				strcmp(srd->key, "skin_toggle") == 0 ||
+				strcmp(srd->key, "skin_open") == 0 ||
+				strcmp(srd->key, "skin_close") == 0 ||
+				strcmp(srd->key, "skin_iconify") == 0) ) )
 				{
 				rd->callbacks = NULL;
 				rd->callbacks_l = 0;
@@ -2734,13 +2732,7 @@ static void ui_edit_copy_ui(UIData *ui, UIData *source)
 	ui_register_free_all(ui);
 	ui_edit_copy_registry(ui, source);
 
-	ui_skin_set(ui, ui_edit_copy_skin(source));
-
-	g_free(ui->skin_path);
-	ui->skin_path = g_strdup(source->skin_path);
-
-	g_free(ui->skin_mode_key);
-	ui->skin_mode_key = g_strdup(source->skin_mode_key);
+	ui_skin_set(ui, ui_edit_copy_skin(source), source->skin_path, source->skin_mode_key);
 }
 
 /*
@@ -3263,8 +3255,18 @@ void ui_edit_set_help_path(EditData *ed, const gchar *path)
 	ed->help_path = g_strdup(path);
 }
 
+static void ui_edit_parent_destroy_cb(GtkWidget *widget, gpointer data)
+{
+	EditData *ed = data;
+	ui_edit_close(ed);
+}
+
 void ui_edit_close(EditData *ed)
 {
+	if (!ed) return;
+
+	gtk_signal_disconnect_by_func(GTK_OBJECT(ed->parent->window), ui_edit_parent_destroy_cb, ed);
+
 	if (ed->widget_window) gtk_widget_destroy(ed->widget_window);
 	if (ed->help_window) gtk_widget_destroy(ed->help_window);
 	gtk_widget_destroy(ed->window);
@@ -3282,7 +3284,7 @@ void ui_edit_close(EditData *ed)
 	g_free(ed);
 }
 
-static gint ui_edit_destroy_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
+static gint ui_edit_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
 {
 	EditData *ed = data;
 	ui_edit_close(ed);
@@ -3335,7 +3337,7 @@ EditData *ui_edit_new(UIData *parent, const gchar *class, const gchar *subclass,
 		gtk_window_set_title(GTK_WINDOW(ed->window), _("Skin editor - SLIK"));
 		}
 
-	gtk_signal_connect(GTK_OBJECT(ed->window), "delete_event", GTK_SIGNAL_FUNC(ui_edit_destroy_cb), ed);
+	gtk_signal_connect(GTK_OBJECT(ed->window), "delete_event", GTK_SIGNAL_FUNC(ui_edit_delete_cb), ed);
 
 	main_vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(ed->window), main_vbox);
@@ -3440,6 +3442,9 @@ EditData *ui_edit_new(UIData *parent, const gchar *class, const gchar *subclass,
 	parent->edit = ed->ui;
 	ed->parent = parent;
 
+	gtk_signal_connect(GTK_OBJECT(parent->window), "destroy",
+			   ui_edit_parent_destroy_cb, ed);
+
 	/* copy ui data from parent */
 
 	ui_edit_copy_ui(ed->ui, parent);
@@ -3580,7 +3585,7 @@ static gint distort_cb(gpointer data)
 
 	if (!ui->skin) return FALSE;
 
-	work = g_list_nth(ui->skin->widget_list, (float)rand() / (float)RAND_MAX * (float)g_list_length(ui->skin->widget_list));
+	work = g_list_nth(ui->skin->widget_list, (float)random() / (float)RAND_MAX * (float)g_list_length(ui->skin->widget_list));
 
 	if (work)
 		{
@@ -3596,8 +3601,8 @@ static gint distort_cb(gpointer data)
 		ox = x;
 		oy = y;
 
-		x += (rand() > RAND_MAX / 2 ? 1 : -1);
-		y += (rand() > RAND_MAX / 2 ? 1 : -1);
+		x += (random() > RAND_MAX / 2 ? 1 : -1);
+		y += (random() > RAND_MAX / 2 ? 1 : -1);
 
 		if (x < 0) x = 0;
 		if (y < 0) y = 0;
@@ -3647,3 +3652,5 @@ void ui_edit_add_skin_resource(const gchar *path)
 
 
 
+
+
