@@ -114,7 +114,7 @@ void list_set_column_attributes(ListData *ld, gint column, gint length, ListColu
 	if (!ld) return;
 	if (column >= ld->column_count) return;
 
-	util_size(&length);
+	if (flags & UI_LIST_COLUMN_SIZE_FIXED) util_size(&length);
 
 	ld->column_widths[column] = length;
 	ld->column_flags[column] = flags;
@@ -129,6 +129,9 @@ void list_image_row(ListData *ld, GdkPixbuf *pb, gint has_press, gint has_prelig
 {
 	if (!ld || !pb) return;
 
+	util_size(&border_left);
+	util_size(&border_right);
+
 	ld->row_overlay = util_size_pixbuf(pb, TRUE);
 	ld->row_height = gdk_pixbuf_get_height(ld->row_overlay) / (1 + (has_press) + (has_prelight));
 	ld->row_width = gdk_pixbuf_get_width(ld->row_overlay);
@@ -240,6 +243,22 @@ static void list_row_free(ListRowData *rd)
 	g_free(rd);
 }
 
+static void list_row_clear_text(ListRowData *rd)
+{
+	gint i;
+
+	if (!rd) return;
+
+	for (i = 0; i < rd->cells; i++)
+		{
+		g_free(rd->text[i]);
+		rd->text[i] = NULL;
+		}
+
+	/* also clear the flag here */
+	rd->flag_mask = 0;
+}
+
 static void list_rows_free(ListData *ld)
 {
 	GList *work;
@@ -437,6 +456,7 @@ static gint list_row_sync(ListData *ld, UIData *ui, const gchar *key, gint row)
 	cd = ui_get_registered_callbacks(ui, key, type_id);
 	if (!cd || !cd->row_info_func) return FALSE;
 
+	list_row_clear_text(rd);
 	return cd->row_info_func(ld, key, row, rd, cd->row_info_data);
 }
 
@@ -456,6 +476,8 @@ static gint list_row_sync_n(ListData *ld, UIData *ui, const gchar *key,
 	while (work && start <= end)
 		{
 		ListRowData *rd = work->data;
+
+		list_row_clear_text(rd);
 		ret |= cd->row_info_func(ld, key, row, rd, cd->row_info_data);
 		work = work->next;
 		start++;
@@ -711,7 +733,7 @@ static void list_draw_rows(ListData *ld, GdkPixbuf *pb, UIData *ui, gint start,
 
 	if (render && ui)
 		{
-		ui_display_render_area(ui, ld->x + ld->border_left, ld->y + start * ld->row_height,
+		ui_display_render_area(ui, ld->x + ld->border_left, ld->y + ld->border_top + start * ld->row_height,
 				       ld->region_width, (end - start + 1) * ld->row_height);
 		}
 }
@@ -972,6 +994,10 @@ static gint list_press(gpointer data, const gchar *key, gint x, gint y, GdkPixbu
 				t = 0;
 				}
 			}
+		else
+			{
+			return FALSE;
+			}
 
 		ld->press_row = row;
 		list_row_press_set(ld, pb, ui, row);
@@ -987,6 +1013,8 @@ static void list_release(gpointer data, const gchar *key, gint x, gint y, GdkPix
 {
 	ListData *ld = data;
 
+	if (!ld->pressed) return;
+
 	if (list_test_proximity(ld, x, y))
 		{
 		gint row;
@@ -1352,7 +1380,7 @@ static void list_refresh_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData
 
 	ld->row_count = length;
 	list_length_sync(ld, length, TRUE);
-	list_row_sync_n(ld, ui, wd->key, 0, ld->row_end- ld->row_start, cd);
+	list_row_sync_n(ld, ui, wd->key, 0, ld->row_end - ld->row_start, cd);
 	list_draw_all_rows(ld, pb, ui, FALSE);
 
 	ui_display_render_area(ui, ld->x + ld->border_left, ld->y + ld->border_top,
@@ -1364,6 +1392,38 @@ gint list_refresh(const gchar *key, UIData *ui)
 	return skin_widget_for_each_key(ui, key, type_id, list_refresh_cb, NULL);
 }
 
+static void list_scroll_row_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui)
+{
+	ListData *ld;
+	gint rows;
+
+	ld = wd->widget;
+	rows = GPOINTER_TO_INT(data);
+
+	list_row_scroll_by_row(ld, pb, ui, wd->key, rows);
+}
+
+gint list_scroll_row(const gchar *key, UIData *ui, gint rows)
+{
+	return skin_widget_for_each_key(ui, key, type_id, list_scroll_row_cb, GINT_TO_POINTER(rows));
+}
+
+static void list_scroll_to_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui)
+{
+	ListData *ld;
+	gint row;
+
+	ld = wd->widget;
+	row = GPOINTER_TO_INT(data);
+
+	list_row_scroll_by_row(ld, pb, ui, wd->key, row - ld->row_start);
+}
+
+gint list_scroll_to(const gchar *key, UIData *ui, gint row)
+{
+	return skin_widget_for_each_key(ui, key, type_id, list_scroll_to_cb, GINT_TO_POINTER(row));
+}
+
 /*
  *-----------------------------
  * internal signals
@@ -1566,3 +1626,4 @@ void list_type_init(void)
 
 
 
+
