@@ -25,6 +25,11 @@
 
 #include "ui2_logo.xpm"
 
+/* these two includes are _only_ used in ui_iconify() */
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+
+
 /*
  *-------------
  * yes, these are globals
@@ -91,6 +96,16 @@ void ui_set_icon(UIData *ui, const char **icon, const gchar *file)
 	window_set_icon(ui->window, icon, file);
 }
 
+void ui_iconify(UIData *ui)
+{
+        Window xwindow;
+
+        if (!ui || !ui->window->window) return;
+
+        xwindow = GDK_WINDOW_XWINDOW(ui->window->window);
+        XIconifyWindow (GDK_DISPLAY (), xwindow, DefaultScreen (GDK_DISPLAY ()));
+}
+
 /*
  *-------------
  * these are the reserved widgets for window resizing and skin mode toggles.
@@ -149,6 +164,36 @@ static void ui_skin_expand_click_cb(ButtonData *button, const gchar *key, gpoint
 		}
 }
 
+static void ui_skin_open_click_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	UIData *ui = data;
+	const gchar *mode_key;
+
+	mode_key = ui_widget_get_data_by_widget(ui, button);
+
+	if (!mode_key || strlen(mode_key) == 0)
+		{
+		printf("ui2_main.c: skin_open requires valid mode key\n");
+		return;
+		}
+
+	if (debug_mode) printf("ui2_main.c: opening skin \"%s\"for addition\n", mode_key);
+}
+
+static void ui_skin_close_click_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	UIData *ui = data;
+
+	if (debug_mode) printf("ui2_main.c: closing skin \"%s\"\n", ui->skin_mode_key);
+}
+
+static void ui_skin_iconify_click_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	UIData *ui = data;
+
+	ui_iconify(ui);
+}
+
 static void ui_add_reserved(UIData *ui)
 {
 	if (!ui_registered_key_exists(ui, "skin_toggle", button_type_id()))
@@ -176,6 +221,31 @@ static void ui_add_reserved(UIData *ui)
 				     NULL, NULL,
 				     NULL, NULL);
 		}
+
+	if (!ui_registered_key_exists(ui, "skin_open", button_type_id()))
+		{
+		button_register_key("skin_open", ui,
+				     NULL, NULL,
+				     ui_skin_open_click_cb, ui,
+				     NULL, NULL,
+				     NULL, NULL);
+		}
+	if (!ui_registered_key_exists(ui, "skin_close", button_type_id()))
+		{
+		button_register_key("skin_close", ui,
+				     NULL, NULL,
+				     ui_skin_close_click_cb, ui,
+				     NULL, NULL,
+				     NULL, NULL);
+		}
+	if (!ui_registered_key_exists(ui, "iconify", button_type_id()))
+		{
+		button_register_key("skin_iconify", ui,
+				     NULL, NULL,
+				     ui_skin_iconify_click_cb, ui,
+				     NULL, NULL,
+				     NULL, NULL);
+		}
 }
 
 /*
@@ -186,6 +256,8 @@ static void ui_add_reserved(UIData *ui)
 
 void ui_free(UIData *ui)
 {
+	if (!ui) return;
+
 	slik_ui_list = g_list_remove(slik_ui_list, ui);
 
 	if (ui->root_win_idle != -1) gtk_idle_remove(ui->root_win_idle);
@@ -197,7 +269,53 @@ void ui_free(UIData *ui)
 	g_free(ui);
 }
 
-UIData *ui_new(const gchar *class, const gchar *subclass, gint decorations, const gchar *title)
+static void ui_show_cb(GtkWidget *widget, gpointer data)
+{
+	UIData *ui = data;
+	gint x, y;
+
+	if (slik_remember_position &&
+	    ui && ui->skin &&
+	    ui_state_get(filename_from_path(ui->skin_path), ui->skin_mode_key, &x, &y, NULL, NULL))
+		{
+		/* ensure that window is at least visible */
+		if (x < 0 - ui->skin->width) x = 0;
+		if (x > gdk_screen_width()) x = gdk_screen_width() - ui->skin->width;
+		if (y < 0 - ui->skin->height) y = 0;
+		if (y > gdk_screen_height()) y = gdk_screen_height() - ui->skin->height;
+
+		if (debug_mode) printf("setting window position %d, %d\n", x, y);
+
+		gdk_window_move(ui->window->window, x, y);
+		}
+
+	/* only do this once */
+	gtk_signal_disconnect_by_func(GTK_OBJECT(widget), ui_show_cb, data);
+}
+
+static void ui_hide_cb(GtkWidget *widget, gpointer data)
+{
+	UIData *ui = data;
+	gint x, y, w, h;
+
+	if (ui_geometry_get(ui, &x, &y, &w, &h))
+		{
+		if (debug_mode) printf("hidden \"%s\"\n", ui->key);
+		ui_state_set(filename_from_path(ui->skin_path), ui->skin_mode_key, x, y, w, h);
+		}
+}
+
+static void ui_destroy_cb(GtkWidget *widget, gpointer data)
+{
+	UIData *ui = data;
+
+	if (debug_mode) printf("UI destroyed \"%s\"\n", ui->key);
+
+	ui_free(ui);
+}
+
+static UIData *ui_real_new(const gchar *class, const gchar *subclass,
+			   gint decorations, const gchar *title, GtkWindowType type)
 {
 	UIData *ui;
 
@@ -213,12 +331,16 @@ UIData *ui_new(const gchar *class, const gchar *subclass, gint decorations, cons
 	ui->key = g_strdup(subclass);
 	ui->allow_move = TRUE;
 
-	ui->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	ui->window = gtk_window_new(type);
 	gtk_window_set_policy(GTK_WINDOW(ui->window), FALSE, FALSE, TRUE);
 	gtk_container_border_width (GTK_CONTAINER(ui->window), 0);
 	gtk_window_set_wmclass(GTK_WINDOW(ui->window), subclass, class);
 	gtk_window_set_title(GTK_WINDOW(ui->window), title);
 
+	gtk_signal_connect(GTK_OBJECT(ui->window), "show", ui_show_cb, ui);
+	gtk_signal_connect(GTK_OBJECT(ui->window), "hide", ui_hide_cb, ui);
+	gtk_signal_connect(GTK_OBJECT(ui->window), "destroy", ui_destroy_cb, ui);
+
 	gtk_widget_realize(ui->window);
 
 	ui->display = gtk_drawing_area_new();
@@ -228,7 +350,7 @@ UIData *ui_new(const gchar *class, const gchar *subclass, gint decorations, cons
 
 	gtk_widget_realize(ui->display);
 
-	if (!ui->decorations) gdk_window_set_decorations(ui->window->window, 0);
+	if (type != GTK_WINDOW_POPUP && !ui->decorations) gdk_window_set_decorations(ui->window->window, 0);
 
 	/* add the default reserved keys */
 	ui_add_reserved(ui);
@@ -238,6 +360,40 @@ UIData *ui_new(const gchar *class, const gchar *subclass, gint decorations, cons
 	return ui;
 }
 
+UIData *ui_new(const gchar *class, const gchar *subclass, gint decorations, const gchar *title)
+{
+	return ui_real_new(class, subclass, decorations, title, GTK_WINDOW_TOPLEVEL);
+}
+
+UIData *ui_new_dialog(const gchar *class, const gchar *subclass, gint decorations, const gchar *title)
+{
+	return ui_real_new(class, subclass, decorations, title, GTK_WINDOW_DIALOG);
+}
+
+static gint ui_close_cb(gpointer data)
+{
+	UIData *ui = data;
+
+	gtk_widget_destroy(ui->window);
+
+	return FALSE;
+}
+
+void ui_close(UIData *ui)
+{
+	if (!ui || ui->destroyed) return;
+
+	/* This may be called in the middle of a widget event (like a button press handler),
+	 * and bad things may happen if the ui vanishes from under the handler, so hide the window
+	 * then clean up in an idle function.
+	 */
+
+	ui->destroyed = TRUE;
+	gtk_widget_hide(ui->window);
+
+	gtk_idle_add(ui_close_cb, ui);
+}
+
 void ui_set_mouse_callback(UIData *ui, void (*func)(UIData *ui, gint button, guint32 time, gpointer data), gpointer data)
 {
 	ui->click_func = func;
@@ -379,6 +535,9 @@ void ui_debug_print_registered_keys(UIData *ui)
 {
 	GList *work;
 
+	printf("UI is \"%s\"\n", ui->skin_path);
+	printf("Key is \"%s\"\n", ui->skin_mode_key);
+
 	printf("UI registered keys (%3d):\n", g_list_length(ui->register_list));
 	printf("=========================\n");
 
@@ -408,7 +567,9 @@ gint ui_geometry_get(UIData *ui, gint *x, gint *y, gint *w, gint *h)
 {
 	gint rx, ry, rw, rh;
 
-	if (!ui || !ui->window || !ui->window->window) return FALSE;
+	if (!ui || !ui->skin ||
+	    !ui->window || !ui->window->window ||
+	    !GTK_WIDGET_VISIBLE(ui->window)) return FALSE;
 
 	gdk_window_get_position(ui->window->window, &rx, &ry);
 	gdk_window_get_size(ui->window->window, &rw, &rh);
@@ -428,8 +589,15 @@ void ui_title_set(UIData *ui, const gchar *text)
 	gtk_window_set_title(GTK_WINDOW(ui->window), text);
 }
 
-void ui_skin_set(UIData *ui, SkinData *skin)
+void ui_skin_set(UIData *ui, SkinData *skin, const gchar *path, const gchar *mode_key)
 {
+	gchar *tmp;
+
+	if (!skin) return;
+
+	/* save current state of old skin */
+	ui_geometry_get(ui, NULL, NULL, NULL, NULL);
+
 	/* remove old internal widget signals */
 	ui_register_free_private(ui);
 
@@ -437,6 +605,16 @@ void ui_skin_set(UIData *ui, SkinData *skin)
 	ui->skin = skin;
 	skin->ui = ui;
 
+	/* use tmp, may be src = dest */
+
+	tmp = g_strdup(path);
+	g_free(ui->skin_path);
+	ui->skin_path = tmp;
+
+	tmp = g_strdup(mode_key);
+	g_free(ui->skin_mode_key);
+	ui->skin_mode_key = tmp;
+
 	/* set up new internal widget signals */
 	skin_widgets_init(skin, ui);
 
@@ -471,6 +649,8 @@ gint ui_skin_load(UIData *ui, const gchar *path, const gchar *mode_key)
 	gchar *cpath;
 	gchar *ckey;
 
+	if (!mode_key) mode_key = "skindata";
+
 	/* copy, since loading a new skin may free the source ? */
 	cpath = g_strdup(path);
 	ckey = g_strdup(mode_key);
@@ -493,10 +673,11 @@ gint ui_skin_load(UIData *ui, const gchar *path, const gchar *mode_key)
 			{
 			gchar *datafile;
 
-			datafile = g_strconcat(cpath, "/", ckey ? ckey : "skindata", NULL);
+			datafile = g_strconcat(cpath, "/", ckey, NULL);
 			skin = skin_parse(cpath, datafile, FALSE);
 			g_free(datafile);
 			}
+
 		}
 	else
 		{
@@ -505,17 +686,7 @@ gint ui_skin_load(UIData *ui, const gchar *path, const gchar *mode_key)
 
 	if (skin)
 		{
-		/* save current state of old skin */
-		ui_geometry_get(ui, NULL, NULL, NULL, NULL);
-
-		g_free(ui->skin_path);
-		ui->skin_path = g_strdup(cpath);
-
-		g_free(ui->skin_mode_key);
-		ui->skin_mode_key = g_strdup(ckey);
-
-		ui_skin_set(ui, skin);
-
+		ui_skin_set(ui, skin, cpath, ckey);
 		success = TRUE;
 		}
 
@@ -580,6 +751,25 @@ gint ui_skin_mode_set(UIData *ui, const gchar *mode_key)
 	return success;
 }
 
+void ui_sync_states(void)
+{
+	GList *work;
+
+	work = slik_ui_list;
+	while (work)
+		{
+		UIData *ui = work->data;
+		gint x, y, w, h;
+
+		if (ui_geometry_get(ui, &x, &y, &w, &h))
+			{
+			ui_state_set(filename_from_path(ui->skin_path), ui->skin_mode_key, x, y, w, h);
+			}
+		
+		work = work->next;
+		}
+}
+
 /*
  *-------------
  * ui_misc text based utils
@@ -748,7 +938,7 @@ GtkWidget *ui_slik_credit(void)
 	gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);
 	gtk_widget_show(pixmap);
 
-	buf = g_strdup_printf("utilizes SLIK %s\nSimpLIstic sKin interface", SLIK_VERSION);
+	buf = g_strdup_printf(_("utilizes SLIK %s\nSimpLIstic sKin interface"), SLIK_VERSION);
 	label = gtk_label_new(buf);
 	g_free(buf);
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
@@ -761,3 +951,6 @@ GtkWidget *ui_slik_credit(void)
 
 
 
+
+
+
