@@ -13,12 +13,17 @@
 #define UI2_MAIN_H
 
 
+/* do not call this to close a ui! use ui_close() instead */
 void ui_free(UIData *ui);
 
 /* subclass is used for the ui's key identifier */
 UIData *ui_new(const gchar *class, const gchar *subclass, gint decorations, const gchar *title);
 void ui_title_set(UIData *ui, const gchar *text);
 
+UIData *ui_new_dialog(const gchar *class, const gchar *subclass, gint decorations, const gchar *title);
+
+void ui_close(UIData *ui);
+
 void ui_set_mouse_callback(UIData *ui, void (*func)(UIData *ui, gint button, guint32 time, gpointer data), gpointer data);
 
 gint ui_geometry_get(UIData *ui, gint *x, gint *y, gint *w, gint *h);
@@ -38,15 +43,17 @@ gint ui_registered_key_exists(UIData *ui, const gchar *key, WidgetType type);
 
 gint ui_widget_exists(UIData *ui, const gchar *key, WidgetType type);
 
-void ui_skin_set(UIData *ui, SkinData *skin);
+void ui_skin_set(UIData *ui, SkinData *skin, const gchar *path, const gchar *mode_key);
 gint ui_skin_load(UIData *ui, const gchar *path, const gchar *mode_key);
 gint ui_skin_mode_set(UIData *ui, const gchar *mode_key);
+void ui_sync_states(void);
 
 void ui_debug_print_registered_keys(UIData *ui);
 void ui_debug_print_all_keys(UIData *ui);
 
 void window_set_icon(GtkWidget *window, const char **icon, const gchar *file);
 void ui_set_icon(UIData *ui, const char **icon, const gchar *file);
+void ui_iconify(UIData *ui);
 
 UIData *ui_find_by_key(const gchar *key);
 
