@@ -192,6 +192,10 @@ static gint skin_parse_main(SkinData *s, GList *list, const gchar *skin_dir, gin
 		if (!key_list_read_int(list, "border_right", &s->border_right)) s->border_right = 1;
 		if (!key_list_read_int(list, "border_top", &s->border_top)) s->border_top = 1;
 		if (!key_list_read_int(list, "border_bottom", &s->border_bottom)) s->border_bottom = 1;
+		util_size(&s->border_left);
+		util_size(&s->border_right);
+		util_size(&s->border_top);
+		util_size(&s->border_bottom);
 
 		s->border_left_stretch = key_list_read_bool(list, "border_left_stretch");
 		s->border_right_stretch = key_list_read_bool(list, "border_right_stretch");
@@ -208,15 +212,35 @@ static gint skin_parse_main(SkinData *s, GList *list, const gchar *skin_dir, gin
 		s->width_def = s->width;
 		s->height_def = s->height;
 		}
+	else
+		{
+		util_size(&s->width_def);
+		util_size(&s->height_def);
+		}
 
 	if (s->sizeable)
 		{
-		if (!key_list_read_int(list, "width_min", &s->width_min)) s->width_min = s->width_def;
-		if (!key_list_read_int(list, "width_max", &s->width_max)) s->width_max = s->width_def;
-		if (!key_list_read_int(list, "width_increment", &s->width_inc)) s->width_inc = 1;
+		if (key_list_read_int(list, "width_min", &s->width_min))
+			util_size(&s->width_min);
+		else
+			s->width_min = s->width_def;
 
-		if (!key_list_read_int(list, "height_min", &s->height_min)) s->height_min = s->height_def;
-		if (!key_list_read_int(list, "height_max", &s->height_max)) s->height_max = s->height_def;
+		if (key_list_read_int(list, "width_max", &s->width_max))
+			util_size(&s->width_max);
+		else
+			s->width_max = s->width_def;
+
+		if (key_list_read_int(list, "height_min", &s->height_min))
+			util_size(&s->height_min);
+		else
+			s->height_min = s->height_def;
+
+		if (key_list_read_int(list, "height_max", &s->height_max))
+			util_size(&s->height_max);
+		else
+			s->height_max = s->height_def;
+
+		if (!key_list_read_int(list, "width_increment", &s->width_inc)) s->width_inc = 1;
 		if (!key_list_read_int(list, "height_increment", &s->height_inc)) s->height_inc = 1;
 		}
 	else
@@ -226,6 +250,9 @@ static gint skin_parse_main(SkinData *s, GList *list, const gchar *skin_dir, gin
 		s->width_inc = s->height_inc = 1;
 		}
 
+	util_size(&s->width_inc);
+	util_size(&s->height_inc);
+
 	s->width = s->width_def;
 	s->height = s->height_def;
 
