@@ -19,7 +19,7 @@ const gchar *key_list_read_chars(GList *list, const gchar *key, gchar **text);
 gchar *key_list_read_path(GList *list, const gchar *key, const gchar *base);
 
 
-SkinData *skin_parse(const gchar *skin_dir, const gchar *datafilei, gint edit);
+SkinData *skin_parse(const gchar *skin_dir, const gchar *datafile, gint edit);
 
 
 #endif
