@@ -137,6 +137,10 @@ void slider_set_attributes(SliderData *slider, gint sizeable,
 {
 	if (!slider) return;
 
+	util_size(&real_length);
+	util_size(&border_1);
+	util_size(&border_2);
+
 	slider->real_length = real_length;
 	slider->border_1 = border_1;
 	slider->border_2 = border_2;
@@ -183,8 +187,13 @@ void slider_sync_overlay(SliderData *slider)
  *-----------------------------
  */
 
-static void slider_force_item_draw_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui)
+static void slider_force_item_draw_cb(WidgetData *wd, gpointer data, UIData *ui)
 {
+	SliderData *slider = data;
+	ItemData *item = wd->widget;
+
+	item->current = item_section_by_percent(item, slider->value);
+
 	wd->hidden = FALSE;
 	ui_widget_draw(ui, wd, FALSE, TRUE);
 	wd->hidden = TRUE;
@@ -264,9 +273,8 @@ static void slider_draw_real(SliderData *slider, gfloat value, gint move,
 
 		if (slider->item_key)
 			{
-			item_section_set_by_percent(slider->item_key, ui, slider->value);
-			skin_widget_for_each_key(ui, slider->item_key, item_type_id(),
-						 slider_force_item_draw_cb, NULL);
+			ui_widget_for_each_key(ui, slider->item_key, item_type_id(),
+					       slider_force_item_draw_cb, slider);
 			}
 
 		if (slider->vertical)
