@@ -28,10 +28,17 @@ void ui_widget_sync_back(UIData *ui, WidgetData *wd);
 void ui_widget_hide(UIData *ui, WidgetData *wd);
 void ui_widget_show(UIData *ui, WidgetData *wd);
 
+void ui_widget_hide_by_key(UIData *ui, const gchar *key, WidgetType type);
+void ui_widget_show_by_key(UIData *ui, const gchar *key, WidgetType type);
+
 gint ui_widget_get_geometry(WidgetData *wd, gint *x, gint *y, gint *w, gint *h);
 void ui_widget_set_coord(UIData *ui, WidgetData *wd, gint x, gint y, gint redraw);
 void ui_widget_set_size(UIData *ui, WidgetData *wd, gint dev_w, gint dev_h, gint redraw);
 
+gint ui_widget_for_each_key(UIData *ui, const gchar *key, WidgetType type,
+                            void (*func)(WidgetData *wd, gpointer data, UIData *ui),
+                            gpointer data);
+
 WidgetData *ui_widget_find_by_coord(UIData *ui, gint x, gint y);
 gint ui_widget_contacts_area(WidgetData *wd, gint x, gint y, gint w, gint h);
 
@@ -51,8 +58,3 @@ WidgetObjectData *ui_widget_object_copy(WidgetObjectData *src);
 #endif
 
 
-
-
-
-
-
