@@ -31,6 +31,7 @@
 
 #include "ui_clist_edit.h"
 #include "ui_fileops.h"
+#include "ui_menu.h"
 #include "ui_utildlg.h"
 
 
@@ -58,11 +59,24 @@ struct _Dest_Data
 	GtkWidget *hidden_button;
 
 	guint32 rename_time;
+	gint right_click_row;
 
 	void (*select_func)(const gchar *path, gpointer data);
 	gpointer select_data;
+
+	GenericDialog *gd;	/* any open confirm dialogs ? */
 };
 
+typedef struct _DestDel_Data DestDel_Data;
+struct _DestDel_Data
+{
+	Dest_Data *dd;
+	gchar *path;
+};
+
+
+static void dest_delete_dlg_cancel(GenericDialog *gd, gpointer data);
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -73,6 +87,14 @@ struct _Dest_Data
 static void dest_free_data(GtkWidget *widget, gpointer data)
 {
 	Dest_Data *dd = data;
+
+	if (dd->gd)
+		{
+		GenericDialog *gd = dd->gd;
+		dest_delete_dlg_cancel(dd->gd, dd->gd->data);
+		generic_dialog_close(gd);
+		}
+
 	g_free(dd->filter);
 	g_free(dd->path);
 	g_free(dd);
@@ -261,6 +283,12 @@ static void dest_change_dir(Dest_Data *dd, const gchar *path, gint retain_name)
 		}
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * destination widget file management utils
+ *-----------------------------------------------------------------------------
+ */ 
+
 static gint dest_row_edit_rename_cb(ClistEditData *ced, const gchar *old, const gchar *new, gpointer data)
 {
 	const gchar *old_path;
@@ -297,36 +325,224 @@ static gint dest_row_edit_rename_cb(ClistEditData *ced, const gchar *old, const
 	return FALSE;
 }
 
+static void dest_delete_dlg_cancel(GenericDialog *gd, gpointer data)
+{
+	DestDel_Data *dl = data;
+
+	dl->dd->gd = NULL;
+	g_free(dl->path);
+	g_free(dl);
+}
+
+static void dest_delete_dlg_ok_cb(GenericDialog *gd, gpointer data)
+{
+	DestDel_Data *dl = data;
+
+	if (unlink (dl->path) < 0)
+		{
+		gchar *text = g_strdup_printf(_("Unable to delete file:\n%s"), dl->path);
+		warning_dialog(_("File deletion failed"), text);
+		g_free(text);
+		}
+	else if (dl->dd->path)
+		{
+		/* refresh list */
+		gchar *path = g_strdup(dl->dd->path);
+		dest_populate(dl->dd, path);
+		g_free(path);
+		}
+
+	dest_delete_dlg_cancel(gd, data);
+}
+
+static void dest_popup_dir_rename_cb(GtkWidget *widget, gpointer data)
+{
+	Dest_Data *dd = data;
+	clist_edit_by_row(GTK_CLIST(dd->d_clist), dd->right_click_row, 0, dest_row_edit_rename_cb, dd);
+}
+
+static void dest_popup_dir_done_cb(GtkWidget *widget, gpointer data)
+{
+	Dest_Data *dd = data;
+	clist_edit_set_highlight(dd->d_clist, dd->right_click_row, FALSE);
+}
+
+static void dest_popup_file_rename_cb(GtkWidget *widget, gpointer data)
+{
+	Dest_Data *dd = data;
+	clist_edit_by_row(GTK_CLIST(dd->f_clist), dd->right_click_row, 0, dest_row_edit_rename_cb, dd);
+}
+
+static void dest_popup_file_delete_cb(GtkWidget *widget, gpointer data)
+{
+	Dest_Data *dd = data;
+	const gchar *path;
+	gchar *text;
+	DestDel_Data *dl;
+
+	path = gtk_clist_get_row_data(GTK_CLIST(dd->f_clist), dd->right_click_row);
+	if (!path) return;
+
+	dl = g_new(DestDel_Data, 1);
+	dl->dd = dd;
+	dl->path = g_strdup(path);
+
+	if (dd->gd)
+		{
+		GenericDialog *gd = dd->gd;
+		dest_delete_dlg_cancel(dd->gd, dd->gd->data);
+		generic_dialog_close(gd);
+		}
+
+	text = g_strdup_printf(_("About to delete the file:\n %s"), path);
+	dd->gd = generic_dialog_new(_("Delete file"), text, "SLIK", "dlg_confirm", TRUE,
+				    dest_delete_dlg_cancel, dl);
+	g_free(text);
+
+	generic_dialog_add(dd->gd, _("Delete"), dest_delete_dlg_ok_cb, TRUE);
+
+	gtk_widget_show(dd->gd->dialog);
+}
+
+static void dest_popup_file_done_cb(GtkWidget *widget, gpointer data)
+{
+	Dest_Data *dd = data;
+	clist_edit_set_highlight(dd->f_clist, dd->right_click_row, FALSE);
+}
+
 static gint dest_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	Dest_Data *dd = data;
 	gint row = -1;
 	gint col = -1;
 
-	if (!event ||
-	    event->button != 1 ||
-	    event->type != GDK_BUTTON_PRESS ||
-	    event->time - dd->rename_time < RENAME_PRESS_DELAY)
+	if (!event) return FALSE;
+	gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y, &row, &col);
+	if (row == -1 || col == -1) return FALSE;
+
+	if (widget == dd->d_clist)
 		{
-		dd->rename_time = event->time;
+		/* dir pressed */
+
+		if (event->button == 3 && row > 1)
+			{
+			/* right click menu */
+
+			GtkWidget *menu;
+
+			dd->right_click_row = row;
+			clist_edit_set_highlight(dd->d_clist, row, TRUE);
+
+			menu = popup_menu_short_lived();
+			menu_item_add(menu, _("Rename"), dest_popup_dir_rename_cb, dd);
+
+			gtk_signal_connect(GTK_OBJECT(menu), "destroy",
+				GTK_SIGNAL_FUNC(dest_popup_dir_done_cb), dd);
+			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, event->time);
+			}
+		
 		return FALSE;
 		}
-
-	gtk_clist_get_selection_info(GTK_CLIST(dd->f_clist), event->x, event->y, &row, &col);
-	if (row == -1 ||
-	    col == -1 ||
-	    !GTK_CLIST(dd->f_clist)->selection ||
-	    row != GPOINTER_TO_INT(GTK_CLIST(dd->f_clist)->selection->data))
+	else
 		{
-		dd->rename_time = event->time;
-		return FALSE;
-		}
+		/* file pressed */
 
-	clist_edit_by_row(GTK_CLIST(dd->f_clist), row, 0, dest_row_edit_rename_cb, NULL);
+		if (event->button == 3)
+			{
+			/* right click menu */
+
+			GtkWidget *menu;
+
+			dd->right_click_row = row;
+			clist_edit_set_highlight(dd->f_clist, row, TRUE);
+
+			menu = popup_menu_short_lived();
+			menu_item_add(menu, _("Rename"), dest_popup_file_rename_cb, dd);
+			menu_item_add(menu, _("Delete"), dest_popup_file_delete_cb, dd);
+
+			gtk_signal_connect(GTK_OBJECT(menu), "destroy",
+				GTK_SIGNAL_FUNC(dest_popup_file_done_cb), dd);
+			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, event->time);
+
+			return FALSE;
+			}
+
+		if (event->button != 1 ||
+		    event->type != GDK_BUTTON_PRESS ||
+		    event->time - dd->rename_time < RENAME_PRESS_DELAY)
+			{
+			dd->rename_time = event->time;
+			return FALSE;
+			}
+
+		if (!GTK_CLIST(dd->f_clist)->selection ||
+		    row != GPOINTER_TO_INT(GTK_CLIST(dd->f_clist)->selection->data))
+			{
+			dd->rename_time = event->time;
+			return FALSE;
+			}
+
+		clist_edit_by_row(GTK_CLIST(dd->f_clist), row, 0, dest_row_edit_rename_cb, dd);
+		}
 
 	return FALSE;
 }
 
+static void dest_new_dir_cb(GtkWidget *widget, gpointer data)
+{
+	Dest_Data *dd = data;
+	gchar *path;
+	gchar *buf;
+	const gchar *tmp;
+	gint from_text = FALSE;
+
+	tmp = gtk_entry_get_text(GTK_ENTRY(dd->entry));
+	if (!isname(tmp))
+		{
+		path = g_strdup(tmp);
+		from_text = TRUE;
+		}
+	else
+		{
+		buf = g_strconcat(dd->path, "/", _("New Directory"), NULL);
+		path = unique_filename(buf, NULL, " ", FALSE);
+		g_free(buf);
+		}
+
+	if (mkdir (path, 0755) < 0)
+		{
+		/* failed */
+		gchar *text;
+
+		text = g_strdup_printf(_("Unable to create directory:\n%s"), filename_from_path(path));
+		warning_dialog(_("Error creating directory"), text);
+		g_free(text);
+
+		g_free(path);
+		}
+	else
+		{
+		gchar *text[2];
+		gint row;
+
+		if (from_text) gtk_entry_set_text(GTK_ENTRY(dd->entry), dd->path);
+
+		text[0] = (gchar *)filename_from_path(path);
+		text[1] = NULL;
+		row = gtk_clist_append(GTK_CLIST(dd->d_clist), text);
+		gtk_clist_set_row_data_full(GTK_CLIST(dd->d_clist), row,
+					    path, (GtkDestroyNotify) g_free);
+
+		clist_edit_by_row(GTK_CLIST(dd->d_clist), row, 0, dest_row_edit_rename_cb, dd);
+		}
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ * destination widget file selection, traversal, view options
+ *-----------------------------------------------------------------------------
+ */ 
+
 static void dest_select_cb(GtkWidget *clist, gint row, gint column,
 			   GdkEventButton *bevent, gpointer data)
 {
@@ -420,7 +636,7 @@ static void dest_filter_list_sync(Dest_Data *dd)
 
 		gtk_combo_set_item_string(GTK_COMBO(dd->filter_combo), GTK_ITEM(cwork->data), filter);
 
-		if (strncmp(old_text, filter, strlen(filter)) == 0)
+		if (strcmp(old_text, filter) == 0)
 			{
 			gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dd->filter_combo)->entry), filter);
 			}
@@ -432,6 +648,31 @@ static void dest_filter_list_sync(Dest_Data *dd)
 	g_free(old_text);
 }
 
+static GList *uig_list_insert_link(GList *list, GList *link, gpointer data)
+{
+	GList *new_list;
+
+	if (!list || link == list) return g_list_prepend(list, data);
+	if (!link) return g_list_append(list, data);
+
+	new_list = g_list_alloc ();
+	new_list->data = data;
+
+	if (link->prev)
+		{
+		link->prev->next = new_list;
+		new_list->prev = link->prev;
+		}
+	else
+		{
+		list = new_list;
+		}
+	link->prev = new_list;
+	new_list->next = link;
+
+	return list;
+}
+
 static void dest_filter_add(Dest_Data *dd, const gchar *filter, const gchar *description, gint set)
 {
 	GList *work;
@@ -454,7 +695,7 @@ static void dest_filter_add(Dest_Data *dd, const gchar *filter, const gchar *des
 		c++;
 		}
 
-	dd->filter_list = g_list_prepend(dd->filter_list, g_strdup(filter));
+	dd->filter_list = uig_list_insert_link(dd->filter_list, g_list_last(dd->filter_list), g_strdup(filter));
 
 	if (description)
 		{
@@ -464,7 +705,7 @@ static void dest_filter_add(Dest_Data *dd, const gchar *filter, const gchar *des
 		{
 		buf = g_strdup_printf("( %s )", filter);
 		}
-	dd->filter_text_list = g_list_prepend(dd->filter_text_list, buf);
+	dd->filter_text_list = uig_list_insert_link(dd->filter_text_list, g_list_last(dd->filter_text_list), buf);
 
 	if (set) gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dd->filter_combo)->entry), filter);
 	dest_filter_list_sync(dd);
@@ -521,6 +762,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	dd->show_hidden = FALSE;
 	dd->select_func = NULL;
 	dd->select_data = NULL;
+	dd->gd = NULL;
 
 	table = gtk_table_new(4, 3, FALSE);
 	gtk_table_set_col_spacing(GTK_TABLE(table), 1, 5);
@@ -539,6 +781,11 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
+	button = gtk_button_new_with_label(_("New Directory"));
+	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(dest_new_dir_cb), dd);
+	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
+
 	dd->hidden_button = gtk_check_button_new_with_label(_("Show hidden"));
 	gtk_signal_connect(GTK_OBJECT(dd->hidden_button), "clicked", GTK_SIGNAL_FUNC(dest_show_hidden_cb), dd);
 	gtk_box_pack_end(GTK_BOX(hbox2), dd->hidden_button, FALSE, FALSE, 0);
@@ -558,8 +805,9 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	dd->d_clist=gtk_clist_new(1);
 #endif
 	gtk_clist_set_column_auto_resize(GTK_CLIST(dd->d_clist), 0, TRUE);
-        gtk_signal_connect (GTK_OBJECT (dd->d_clist), "select_row",(GtkSignalFunc) dest_select_cb, dd);
-	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "destroy", (GtkSignalFunc) dest_free_data, dd);
+	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "button_press_event", (GtkSignalFunc)dest_press_cb, dd);
+        gtk_signal_connect(GTK_OBJECT(dd->d_clist), "select_row", (GtkSignalFunc)dest_select_cb, dd);
+	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "destroy", (GtkSignalFunc)dest_free_data, dd);
 	gtk_widget_set_usize(dd->d_clist, DEST_WIDTH, DEST_HEIGHT);
 	gtk_container_add (GTK_CONTAINER (scrolled), dd->d_clist);
         gtk_widget_show (dd->d_clist);
@@ -596,10 +844,10 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 #endif
 		gtk_clist_set_column_auto_resize(GTK_CLIST(dd->f_clist), 0, TRUE);
 		gtk_widget_set_usize(dd->f_clist, DEST_WIDTH, DEST_HEIGHT);
-		gtk_signal_connect (GTK_OBJECT (dd->f_clist), "select_row",(GtkSignalFunc) dest_select_cb, dd);
-		gtk_signal_connect (GTK_OBJECT (dd->f_clist), "button_press_event",(GtkSignalFunc) dest_press_cb, dd);
-		gtk_container_add (GTK_CONTAINER (scrolled), dd->f_clist);
-	        gtk_widget_show (dd->f_clist);
+		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "button_press_event", (GtkSignalFunc)dest_press_cb, dd);
+		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "select_row",(GtkSignalFunc) dest_select_cb, dd);
+		gtk_container_add(GTK_CONTAINER (scrolled), dd->f_clist);
+	        gtk_widget_show(dd->f_clist);
 
 		dest_filter_clear(dd);
 		dest_filter_add(dd, filter, filter_desc, TRUE);
@@ -692,3 +940,5 @@ void path_selection_clear_filter(GtkWidget *entry)
 	dest_filter_clear(dd);
 }
 
+
+
